#
# Copyright (c) 2014 Juniper Networks, Inc. All rights reserved.
#

import uuid

from neutron.extensions import loadbalancer

from neutron.plugins.common import constants
from neutron.services import provider_configuration as pconf

from neutron.openstack.common import uuidutils
from vnc_api.vnc_api import IdPermsType, NoIdError, HttpError
from vnc_api.vnc_api import LoadbalancerPool, LoadbalancerPoolType

from resource_manager import ResourceManager
from resource_manager import LoadbalancerMethodInvalid

class LoadbalancerPoolManager(ResourceManager):

    _loadbalancer_pool_type_mapping = {
        'admin_state': 'admin_state_up',
        'protocol': 'protocol',
        'loadbalancer_method': 'lb_method',
        'subnet_id': 'subnet_id'
    }

    @property
    def property_type_mapping(self):
        return self._loadbalancer_pool_type_mapping

    def make_properties(self, pool):
        props = LoadbalancerPoolType()
        for key, mapping in self._loadbalancer_pool_type_mapping.iteritems():
            if mapping in pool:
                setattr(props, key, pool[mapping])
        return props

    def make_dict(self, pool, fields=None):
        res = {
            'id': pool.uuid,
            'tenant_id': pool.parent_uuid.replace('-', ''),
            'name': pool.display_name,
            'description': self._get_object_description(pool),
            'status': self._get_object_status(pool),
        }

        props = pool.get_loadbalancer_pool_properties()
        for key, mapping in self._loadbalancer_pool_type_mapping.iteritems():
            value = getattr(props, key, None)
            if value is not None:
                res[mapping] = value

        res['provider'] = pool.get_loadbalancer_pool_provider()

        # vip_id
        res['vip_id'] = None
        vip_refs = pool.get_virtual_ip_back_refs()
        if vip_refs is not None:
            res['vip_id'] = vip_refs[0]['uuid']

        # members
        res['members'] = []
        members = pool.get_loadbalancer_members()
        if members is not None:
            res['members'] = [member['uuid'] for member in members]

        # health_monitors
        res['health_monitors'] = []
        hm_refs = pool.get_loadbalancer_healthmonitor_refs()
        if hm_refs is not None:
            res['health_monitors'] = [hm['uuid'] for hm in hm_refs]

        # TODO: health_monitor_status
        res['health_monitors_status'] = []

        return self._fields(res, fields)

    def resource_read(self, id):
        return self._api.loadbalancer_pool_read(id=id)

    def resource_list(self, tenant_id=None):
        if tenant_id:
            parent_id = str(uuid.UUID(tenant_id))
        else:
            parent_id = None
        return self._api.loadbalancer_pools_list(parent_id=parent_id)

    def resource_update(self, obj):
        try:
            return self._api.loadbalancer_pool_update(obj)
        except HttpError as e:
            if 'LoadbalancerMethodType' in e.content:
                pool_props = obj.get_loadbalancer_pool_properties()
                lb_method = pool_props.get_loadbalancer_method()
                raise LoadbalancerMethodInvalid(lb_method=lb_method,
                                                pool_id=obj.uuid)

    def resource_delete(self, id):
        return self._api.loadbalancer_pool_delete(id=id)

    def get_exception_notfound(self, id=None):
        return loadbalancer.PoolNotFound(pool_id=id)

    def get_exception_inuse(self, id=None):
        return loadbalancer.PoolInUse(pool_id=id)

    @property
    def neutron_name(self):
        return "pool"

    @property
    def resource_name_plural(self):
        return "loadbalancer-pools"

    def create(self, context, pool):
        """
        Create a loadbalancer_pool object.
        """
        p = pool['pool']
        try:
            sas_fq_name = ["default-global-system-config"]
            sas_fq_name.append(p['provider'])
            sas_obj = self._api.service_appliance_set_read(fq_name=sas_fq_name)
        except NoIdError:
            raise pconf.ServiceProviderNotFound(
                provider=p['provider'], service_type=constants.LOADBALANCER)

        tenant_id = self._get_tenant_id_for_create(context, p)
        project = self._project_read(project_id=tenant_id)

        uuid = uuidutils.generate_uuid()
        name = self._get_resource_name('loadbalancer-pool', project,
                                       p['name'], uuid)
        props = self.make_properties(p)
        id_perms = IdPermsType(enable=True,
                               description=p['description'])
        pool = LoadbalancerPool(name, project,
                                loadbalancer_pool_properties=props,
                                loadbalancer_pool_provider=p['provider'],
                                id_perms=id_perms, display_name=p['name'])
        pool.uuid = uuid

        if p['health_monitors']:
            for hm in p['health_monitors']:
                try:
                    mon = self._api.loadbalancer_healthmonitor_read(id=hm)
                except NoIdError:
                    raise loadbalancer.HealthMonitorNotFound(monitor_id=hm)
                pool.add_loadbalancer_healthmonitor(mon)

        pool.set_service_appliance_set(sas_obj)

        self._api.loadbalancer_pool_create(pool)
        return self.make_dict(pool)

    def update_properties(self, pool_db, id, p):
        props = pool_db.get_loadbalancer_pool_properties()
        if self.update_properties_subr(props, p):
            pool_db.set_loadbalancer_pool_properties(props)
            return True
        return False



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2014 Juniper Networks.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Hampapur Ajay, Praneet Bachheti

import copy
import logging
from pprint import pformat
import sys

import cgitb

LOG = logging.getLogger(__name__)


class NeutronPluginContrailPolicy(object):
    def set_core(self, core_instance):
        self._core = core_instance

    def _make_policy_dict(self, entry, status_code=None, fields=None):
        return entry

    def create_policy(self, context, policy):
        """
        Creates a new Policy, and assigns it a symbolic name.
        """
        plugin_policy = copy.deepcopy(policy)

        policy_dicts = self._core._create_resource('policy', context,
                                                   plugin_policy)
        LOG.debug("create_policy(): " + pformat(policy_dicts) + "\n")

        return policy_dicts

    def get_policy(self, context, policy_id, fields=None):
        """
        Get the attributes of a policy.
        """
        policy_dicts = self._core._get_resource('policy', context, policy_id,
                                                fields)

        LOG.debug("get_policy(): " + pformat(policy_dicts))
        return policy_dicts

    def update_policy(self, context, policy_id, policy):
        """
        Updates the attributes of a particular policy.
        """
        plugin_policy = copy.deepcopy(policy)
        policy_dicts = self._core._update_resource('policy', context,
                                                   policy_id, plugin_policy)

        LOG.debug("update_policy(): " + pformat(policy_dicts))
        return policy_dicts

    def delete_policy(self, context, policy_id):
        """
        Deletes the Policy with the specified identifier
        """
        self._core._delete_resource('policy', context, policy_id)

        LOG.debug("delete_policy(): %s" % (policy_id))

    def get_policys(self, context, filters=None, fields=None):
        """
        Retrieves all policies identifiers.
        """
        policy_dicts = self._core._list_resource('policy', context, filters,
                                                 fields)

        LOG.debug(
            "get_policys(): filters: " + pformat(filters) + " data: "
            + pformat(policy_dicts))
        return policy_dicts

    def get_policy_count(self, context, filters=None):
        """
        Get the count of policies.
        """
        policies_count = self._core._count_resource('policy', context, filters)

        LOG.debug("get_policy_count(): filters: " + pformat(filters) +
                  " data: " + str(policies_count['count']))
        return policies_count['count']



#
# Copyright (c) 2014 Juniper Networks, Inc. All rights reserved.
#
from neutron.api.v2 import attributes
from neutron.common import exceptions as n_exc
from neutron.extensions import loadbalancer
from neutron.openstack.common import log as logging
from neutron.openstack.common import uuidutils
from vnc_api.vnc_api import IdPermsType, NoIdError
from vnc_api.vnc_api import InstanceIp, VirtualMachineInterface
from vnc_api.vnc_api import SecurityGroup
from vnc_api.vnc_api import VirtualIp, VirtualIpType

from resource_manager import ResourceManager
import utils
import uuid

LOG = logging.getLogger(__name__)


class VirtualIpManager(ResourceManager):
    _virtual_ip_type_mapping = {
        'address': 'address',
        'protocol': 'protocol',
        'protocol_port': 'protocol_port',
        'connection_limit': 'connection_limit',
        'subnet_id': 'subnet_id',
        'admin_state': 'admin_state_up',
    }

    @property
    def property_type_mapping(self):
        return self._virtual_ip_type_mapping

    def make_properties(self, vip):
        props = VirtualIpType()
        for key, mapping in self._virtual_ip_type_mapping.iteritems():
            if mapping in vip and vip[mapping] != attributes.ATTR_NOT_SPECIFIED:
                setattr(props, key, vip[mapping])

        sp = vip['session_persistence']
        if sp is not None:
            if 'type' in sp:
                props.persistence_type = sp['type']
            if 'cookie_name' in sp:
                props.persistence_cookie_name = sp['cookie_name']
        return props

    def _get_vip_pool_id(self, vip):
        pool_refs = vip.get_loadbalancer_pool_refs()
        if pool_refs is None:
            return None
        return pool_refs[0]['uuid']

    def _get_interface_params(self, vip, props):
        vmi_list = vip.get_virtual_machine_interface_refs()
        if vmi_list is None:
            return None

        port_id = vmi_list[0]['uuid']
        if not props.address or props.address == attributes.ATTR_NOT_SPECIFIED:
            try:
                vmi = self._api.virtual_machine_interface_read(id=port_id)
            except NoIdError as ex:
                LOG.error(ex)
                return None

            ip_refs = vmi.get_instance_ip_back_refs()
            if ip_refs:
                try:
                    iip = self._api.instance_ip_read(id=ip_refs[0]['uuid'])
                except NoIdError as ex:
                    LOG.error(ex)
                    return None
                props.address = iip.get_instance_ip_address()

        return port_id

    def make_dict(self, vip, fields=None):
        props = vip.get_virtual_ip_properties()
        port_id = self._get_interface_params(vip, props)

        res = {'id': vip.uuid,
               'tenant_id': vip.parent_uuid.replace('-', ''),
               'name': vip.display_name,
               'description': self._get_object_description(vip),
               'subnet_id': props.subnet_id,
               'address': props.address,
               'port_id': port_id,
               'protocol_port': props.protocol_port,
               'protocol': props.protocol,
               'pool_id': self._get_vip_pool_id(vip),
               'session_persistence': None,
               'connection_limit': props.connection_limit,
               'admin_state_up': props.admin_state,
               'status': self._get_object_status(vip)}

        if props.persistence_type:
            sp = {'type': props.persistence_type}
            if props.persistence_type == 'APP_COOKIE':
                sp['cookie_name'] = props.persistence_cookie_name
            res['session_persistence'] = sp

        return self._fields(res, fields)

    def resource_read(self, id):
        return self._api.virtual_ip_read(id=id)

    def resource_list(self, tenant_id=None):
        if tenant_id:
            parent_id = str(uuid.UUID(tenant_id))
        else:
            parent_id = None
        return self._api.virtual_ips_list(parent_id=parent_id)

    def resource_update(self, obj):
        return self._api.virtual_ip_update(obj)

    def resource_delete(self, id):
        return self._api.virtual_ip_delete(id=id)

    def get_exception_notfound(self, id=None):
        return loadbalancer.VipNotFound(vip_id=id)

    def get_exception_inuse(self, id=None):
        pass

    @property
    def neutron_name(self):
        return "vip"

    @property
    def resource_name_plural(self):
        return "virtual-ips"

    def _create_virtual_interface(self, project, vip_id, subnet_id,
                                  ip_address):
        network_id = utils.get_subnet_network_id(self._api, subnet_id)
        try:
            vnet = self._api.virtual_network_read(id=network_id)
        except NoIdError:
            raise n_exc.NetworkNotFound(net_id=network_id)

        vmi = VirtualMachineInterface(vip_id, project)
        vmi.set_virtual_network(vnet)

        sg_obj = SecurityGroup("default", project)
        vmi.add_security_group(sg_obj)
        self._api.virtual_machine_interface_create(vmi)

        iip_obj = InstanceIp(name=vip_id)
        iip_obj.set_virtual_network(vnet)
        iip_obj.set_virtual_machine_interface(vmi)
        if ip_address and ip_address != attributes.ATTR_NOT_SPECIFIED:
            iip_obj.set_instance_ip_address(ip_address)
        self._api.instance_ip_create(iip_obj)
        iip = self._api.instance_ip_read(id=iip_obj.uuid)
        vip_address = iip.get_instance_ip_address()

        return vmi, vip_address

    def _delete_virtual_interface(self, vmi_list):
        if vmi_list is None:
            return

        for vmi_ref in vmi_list:
            interface_id = vmi_ref['uuid']
            try:
                vmi = self._api.virtual_machine_interface_read(id=interface_id)
            except NoIdError as ex:
                LOG.error(ex)
                continue

            ip_refs = vmi.get_instance_ip_back_refs()
            if ip_refs:
                for ref in ip_refs:
                    self._api.instance_ip_delete(id=ref['uuid'])

            fip_refs = vmi.get_floating_ip_back_refs()
            for ref in fip_refs or []:
                try:
                    fip = self._api.floating_ip_read(id=ref['uuid'])
                except NoIdError as ex:
                    LOG.error(ex)
                    continue
                fip.set_virtual_machine_interface_list([])
                self._api.floating_ip_update(fip)

            self._api.virtual_machine_interface_delete(id=interface_id)

    def create(self, context, vip):
        """
        Create a VIP.
        """
        v = vip['vip']
        tenant_id = self._get_tenant_id_for_create(context, v)

        project = self._project_read(project_id=tenant_id)

        if v['pool_id']:
            try:
                pool = self._api.loadbalancer_pool_read(id=v['pool_id'])
            except NoIdError:
                raise loadbalancer.PoolNotFound(pool_id=v['pool_id'])
            project_id = pool.parent_uuid
            if str(uuid.UUID(tenant_id)) != project_id:
                raise n_exc.NotAuthorized()
            protocol = pool.get_loadbalancer_pool_properties().get_protocol()
            if protocol != v['protocol']:
                raise loadbalancer.ProtocolMismatch(
                    vip_proto=v['protocol'], pool_proto=protocol)
            if pool.get_virtual_ip_back_refs():
                raise loadbalancer.VipExists(pool_id=v['pool_id'])
        else:
            pool = None

        obj_uuid = uuidutils.generate_uuid()
        name = self._get_resource_name('virtual-ip', project, v['name'], obj_uuid)
        id_perms = IdPermsType(enable=True,
                               description=v['description'])
        vip = VirtualIp(name, project, id_perms=id_perms,
                        display_name=v['name'])
        vip.uuid = obj_uuid

        if pool:
            vip.set_loadbalancer_pool(pool)

        vmi, vip_address = self._create_virtual_interface(project,
            obj_uuid, v['subnet_id'], v.get('address'))
        vip.set_virtual_machine_interface(vmi)

        props = self.make_properties(v)
        props.set_address(vip_address)
        vip.set_virtual_ip_properties(props)

        self._api.virtual_ip_create(vip)
        return self.make_dict(vip)

    def delete(self, context, id):
        try:
            vip = self._api.virtual_ip_read(id=id)
        except NoIdError:
            loadbalancer.VipNotFound(vip_id=id)

        super(VirtualIpManager, self).delete(context, id)
        self._delete_virtual_interface(
            vip.get_virtual_machine_interface_refs())

    def _update_virtual_ip_properties(self, props, id, vip):
        """
        Update virtual ip properties and return True if the have been
        modified
        """
        # according to the spec:
        # status, subnet_id, address, port and protocol are immutable
        immutable = ['address', 'protocol', 'protocol_port', 'subnet_id']
        for field in immutable:
            if field not in vip:
                continue
            if getattr(props, field) != vip[field]:
                msg = 'Attribute %s in vip %s is immutable' % (field, id)
                raise n_exc.BadRequest(resource='vip', msg=msg)

        # update
        change = self.update_properties_subr(props, vip)

        if 'session_persistence' in vip:
            sp = vip['session_persistence']

            new_type = sp.get('type', None)
            if props.persistence_type != new_type:
                props.persistence_type = new_type
                change = True
            new_cookie_name = sp.get('cookie_name', None)
            if props.persistence_cookie_name != new_cookie_name and \
                    props.persistence_type == 'APP_COOKIE':
                props.persistence_cookie_name = new_cookie_name
                change = True

        return change

    def update_properties(self, vip_db, id, v):
        props = vip_db.get_virtual_ip_properties()
        if self._update_virtual_ip_properties(props, id, v):
            vip_db.set_virtual_ip_properties(props)
            return True
        return False

    def update_object(self, vip_db, id, v):
        if 'pool_id' in v and self._get_vip_pool_id(vip_db) != v['pool_id']:
            try:
                pool = self._api.loadbalancer_pool_read(id=v['pool_id'])
            except NoIdError:
                raise loadbalancer.PoolNotFound(pool_id=v['pool_id'])
            if vip_db.parent_uuid != pool.parent_uuid:
                raise n_exc.NotAuthorized()

            # check that the pool has no vip configured
            if pool.get_virtual_ip_back_refs():
                raise loadbalancer.VipExists(pool_id=pool_obj.uuid)

            # check that the protocol matches
            pool_props = pool.get_loadbalancer_pool_properties()
            vip_props = vip_db.get_virtual_ip_properties()
            if pool_props.get_protocol() != vip_props.get_protocol():
                raise loadbalancer.ProtocolMismatch(
                    vip_proto=vip_props.get_protocol(),
                    pool_proto=pool_props.get_protocol())

            # update vip
            vip_db.set_loadbalancer_pool(pool)
            return True

        return False



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2014 Juniper Networks.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Hampapur Ajay, Praneet Bachheti

import copy
import logging
from pprint import pformat
import sys

LOG = logging.getLogger(__name__)


class NeutronPluginContrailVpc(object):
    def set_core(self, core_instance):
        self._core = core_instance

    # VPC route table handlers
    def _make_route_table_routes_dict(self, route_table_route, fields=None):
        res = {'prefix': route_table_route['prefix'],
               'next_hop': route_table_route['next_hop']}

        return self._core._fields(res, fields)

    def _make_route_table_dict(self, route_table, status_code=None,
                               fields=None):
        res = {'id': route_table['id'],
               'name': route_table['name'],
               'fq_name': route_table['fq_name'],
               'tenant_id': route_table['tenant_id']}
        if route_table['routes']:
            res['routes'] = [self._make_route_table_routes_dict(r)
                             for r in route_table['routes']['route']]
        else:
            res['routes'] = {}
        return self._core._fields(res, fields)

    def create_route_table(self, context, route_table):
        """
        Creates a Route Table.
        """
        plugin_rt = copy.deepcopy(route_table)

        rt_dicts = self._core._create_resource('route_table', context,
                                               plugin_rt)
        LOG.debug("create_route_table(): " + pformat(rt_dicts) + "\n")

        return rt_dicts

    def get_route_table(self, context, rt_id, fields=None):
        """
        Get the attributes of a route table.
        """
        rt_dicts = self._core._get_resource('route_table', context, rt_id,
                                            fields)

        LOG.debug("get_route_table(): " + pformat(rt_dicts))

        return rt_dicts

    def update_route_table(self, context, rt_id, route_table):
        """
        Updates the attributes of a particular route table.
        """
        plugin_rt = copy.deepcopy(route_table)
        rt_dicts = self._core._update_resource('route_table', context, rt_id,
                                               plugin_rt)

        LOG.debug("update_route_table(): " + pformat(rt_dicts))
        return rt_dicts

    def delete_route_table(self, context, rt_id):
        """
        Deletes a route table
        """
        self._core._delete_resource('route_table', context,
                                                    rt_id)

        LOG.debug("delete_security_group(): %s" % (rt_id))

    def get_route_tables(self, context, filters=None, fields=None,
                         sorts=None, limit=None, marker=None,
                         page_reverse=False):
        """
        Retrieves all route tables
        """
        rt_dicts = self._core._list_resource('route_table', context, filters,
                                             fields)

        LOG.debug(
            "get_route_tables(): filters: " + pformat(filters) + " data: "
            + pformat(rt_dicts))
        return rt_dicts

    # VPC route table nat instance handlers
    def _make_nat_instance_dict(self, nat_instance, status_code=None,
                                fields=None):
        res = {'id': nat_instance['id'],
               'name': nat_instance['name'],
               'tenant_id': nat_instance['tenant_id']}
        if nat_instance['internal_net']:
            res['internal_net'] = nat_instance['internal_net']
        if nat_instance['external_net']:
            res['external_net'] = nat_instance['external_net']
        return self._core._fields(res, fields)

    def create_nat_instance(self, context, nat_instance):
        """
        Creates a new nat instance
        """
        plugin_nat = copy.deepcopy(nat_instance)

        nat_dicts = self._core._create_resource('nat_instance', context,
                                                plugin_nat)

        LOG.debug("create_nat_instance(): " + pformat(nat_dicts) + "\n")
        return nat_dicts

    def get_nat_instance(self, context, nat_id, fields=None):
        """
        Get the attributes of a particular nat instance
        """
        nat_dicts = self._core._get_resource('nat_instance', context, nat_id,
                                             fields)

        LOG.debug("get_nat_instance(): " + pformat(nat_dicts))
        return self._core._fields(nat_dicts, fields)

    def delete_nat_instance(self, context, nat_id):
        """
        Deletes the nat instance
        """
        self._core._delete_resource('nat_instance', context, nat_id)

        LOG.debug("delete_nat_instance(): %s" % (nat_id))

    def get_nat_instances(self, context, filters=None, fields=None,
                          sorts=None, limit=None, marker=None,
                          page_reverse=False):
        """
        Get the list of nat instances.
        """
        nat_dicts = self._core._list_resource('nat_instance', context,
                                              filters, fields)

        LOG.debug(
            "get_nat_instances(): filters: " + pformat(filters) + " data: "
            + pformat(nat_dicts))
        return nat_dicts



#
# Copyright (c) 2014 Juniper Networks, Inc. All rights reserved.
#
from loadbalancer_db import LoadBalancerPluginDb

class LoadBalancerPlugin(LoadBalancerPluginDb):
    supported_extension_aliases = ["lbaas"]

    def __init__(self):
        super(LoadBalancerPlugin, self).__init__()

    def get_plugin_description(self):
        return "OpenContrail LoadBalancer Service Plugin"

    def _pool_update_provider(self, context, pool):
        if 'provider' not in pool or pool['provider'].__class__ is object:
            pool['provider'] = "opencontrail"

    def create_pool(self, context, pool):
        self._pool_update_provider(context, pool['pool'])
        p = super(LoadBalancerPlugin, self).create_pool(context, pool)
	return p





#
# Copyright (c) 2014 Juniper Networks, Inc. All rights reserved.
#

import uuid

from neutron.extensions import loadbalancer
from neutron.openstack.common import uuidutils
from vnc_api.vnc_api import IdPermsType, NoIdError
from vnc_api.vnc_api import LoadbalancerMember, LoadbalancerMemberType

from resource_manager import ResourceManager


class LoadbalancerMemberManager(ResourceManager):
    _loadbalancer_member_type_mapping = {
        'admin_state': 'admin_state_up',
        'status': 'status',
        'protocol_port': 'protocol_port',
        'weight': 'weight',
        'address': 'address',
    }

    @property
    def property_type_mapping(self):
        return self._loadbalancer_member_type_mapping

    def make_properties(self, member):
        props = LoadbalancerMemberType()
        for key, mapping in self._loadbalancer_member_type_mapping.iteritems():
            if mapping in member:
                setattr(props, key, member[mapping])
        return props

    def _get_member_pool_id(self, member):
        pool_uuid = member.parent_uuid
        return pool_uuid

    def make_dict(self, member, fields=None):
        res = {'id': member.uuid,
               'pool_id': member.parent_uuid,
               'status': self._get_object_status(member)}

        try:
            pool = self._api.loadbalancer_pool_read(id=member.parent_uuid)
            res['tenant_id'] = pool.parent_uuid.replace('-', '')
        except NoIdError:
            pass

        props = member.get_loadbalancer_member_properties()
        for key, mapping in self._loadbalancer_member_type_mapping.iteritems():
            value = getattr(props, key, None)
            if value is not None:
                res[mapping] = value

        return self._fields(res, fields)

    def resource_read(self, id):
        return self._api.loadbalancer_member_read(id=id)

    def resource_list(self, tenant_id=None):
        """ In order to retrive all the members for a specific tenant
        the code iterates through all the pools.
        """
        if tenant_id is None:
            return self._api.loadbalancer_members_list()

        pool_list = self._api.loadbalancer_pools_list(tenant_id)
        if 'loadbalancer-pools' not in pool_list:
            return {}

        member_list = []
        for pool in pool_list['loadbalancer-pools']:
            pool_members = self._api.loadbalancer_members_list(
                parent_id=pool['uuid'])
            if 'loadbalancer-members' in pool_members:
                member_list.extend(pool_members['loadbalancer-members'])

        response = {'loadbalancer-members': member_list}
        return response

    def get_collection(self, context, filters=None, fields=None):
        """ Optimize the query for members in a pool.
        """
        if 'pool_id' not in filters:
            return super(LoadbalancerMemberManager, self).get_collection(
                context, filters, fields)

        member_list = []
        for pool in filters['pool_id']:
            pool_members = self._api.loadbalancer_members_list(
                parent_id=pool)
            if 'loadbalancer-members' in pool_members:
                member_list.extend(pool_members['loadbalancer-members'])

        response = []
        for m in member_list:
            res = self._get_resource_dict(m['uuid'], filters, fields)
            if res is not None and self._is_authorized(context, res):
                response.append(res)

        return response

    def resource_update(self, obj):
        return self._api.loadbalancer_member_update(obj)

    def resource_delete(self, id):
        return self._api.loadbalancer_member_delete(id=id)

    def get_exception_notfound(self, id=None):
        return loadbalancer.MemberNotFound(member_id=id)

    def get_exception_inuse(self, id=None):
        pass

    @property
    def neutron_name(self):
        return "member"

    @property
    def resource_name_plural(self):
        return "loadbalancer-members"

    def create(self, context, member):
        """
        Create a loadbalancer_member object.
        """
        m = member['member']
        try:
            pool = self._api.loadbalancer_pool_read(id=m['pool_id'])
        except NoIdError:
            raise loadbalancer.PoolNotFound(pool_id=m['pool_id'])

        tenant_id = self._get_tenant_id_for_create(context, m)
        if str(uuid.UUID(tenant_id)) != pool.parent_uuid:
            raise n_exc.NotAuthorized()

        obj_uuid = uuidutils.generate_uuid()
        props = self.make_properties(m)
        id_perms = IdPermsType(enable=True)

        member_db = LoadbalancerMember(
            obj_uuid, pool, loadbalancer_member_properties=props,
            id_perms=id_perms)
        member_db.uuid = obj_uuid

        self._api.loadbalancer_member_create(member_db)
        return self.make_dict(member_db)

    def update_properties(self, member_db, id, m):
        props = member_db.get_loadbalancer_member_properties()
        if self.update_properties_subr(props, m):
            member_db.set_loadbalancer_member_properties(props)
            return True
        return False

    def update_object(self, member_db, id, m):
        if 'pool_id' in m and self._get_member_pool_id(member_db) != m['pool_id']:
            try:
                pool = self._api.loadbalancer_pool_read(id=m['pool_id'])
            except NoIdError:
                raise loadbalancer.PoolNotFound(pool_id=m['pool_id'])

            db_props = member_db.get_loadbalancer_member_properties()
            members = pool.get_loadbalancer_members()
            for member in members or []:
                member_obj = self._api.loadbalancer_member_read(
                    id=member['uuid'])
                props = member_obj.get_loadbalancer_member_properties()
                if ((props.get_address() == db_props.get_address()) and
                    (props.get_protocol_port() == db_props.get_protocol_port())):
                    raise loadbalancer.MemberExists(
                        address=props.get_address(),
                        port=props.get_protocol_port(),
                        pool=m['pool_id'])

        # delete member from old pool
        props = member_db.get_loadbalancer_member_properties()
        obj_uuid = member_db.uuid
        self._api.loadbalancer_member_delete(id=member_db.uuid)

        # create member for the new pool with same uuid and props
        id_perms = IdPermsType(enable=True)
        member_obj = LoadbalancerMember(
            obj_uuid, pool, loadbalancer_member_properties=props,
            id_perms=id_perms)
        member_obj.uuid = obj_uuid
        self._api.loadbalancer_member_create(member_obj)

        return True



#
# Copyright (c) 2014 Juniper Networks, Inc. All rights reserved.
#

from abc import ABCMeta, abstractmethod, abstractproperty
from eventlet import greenthread
from neutron.common import exceptions as n_exc
from neutron.extensions import loadbalancer
from neutron.plugins.common import constants
from vnc_api.vnc_api import NoIdError, RefsExistError
import six
import uuid

class LoadbalancerMethodInvalid(n_exc.BadRequest):
    message = _("Method %(lb_method)s not supported for pool %(pool_id)s")


@six.add_metaclass(ABCMeta)
class ResourceManager(object):
    _max_project_read_attempts = 3

    def __init__(self, api):
        self._api = api

    @abstractproperty
    def property_type_mapping(self):
        """ Mapping from property name to neutron dict key.
        """
        pass

    @abstractmethod
    def make_properties(self, resource):
        """ Returns the properties for the specified resource.
        """
        pass

    @abstractmethod
    def make_dict(self, resource, fields):
        """ Return the contrail api resource in the dictionary format
        expected by neutron.
        """
        pass

    @abstractmethod
    def resource_read(self, id):
        """ Read the specified resource from the api server.
        """
        pass

    @abstractmethod
    def resource_list(self, tenant_id):
        """ Returns the list of objects from the api server.
        """
        pass

    @abstractmethod
    def resource_update(self, obj):
        """ Call the update method.
        """
        pass

    @abstractmethod
    def resource_delete(self, id):
        """ Delete the specified resource from the api server.
        """
        pass

    @abstractproperty
    def get_exception_notfound(self, id):
        """ Returns the correct NotFound exception.
        """
        pass

    @abstractproperty
    def get_exception_inuse(self, id):
        """ Returns the correct NotFound exception.
        """
        pass

    @abstractproperty
    def neutron_name(self):
        """ Resource name in a request from neutron.
        """
        pass

    @abstractproperty
    def resource_name_plural(self):
        """ Resource list name in a list response from api server.
        """
        pass

    @abstractmethod
    def create(self, context, resource):
        """ Create resource.
        """
        pass

    @abstractmethod
    def update_properties(self, obj, id, resource):
        """ Update the resource properties
        """
        return False

    def update_object(self, obj, id, resource):
        """ Update object metadata other than properties
        """
        return False

    def _get_tenant_id_for_create(self, context, resource):
        if context.is_admin and 'tenant_id' in resource:
            tenant_id = resource['tenant_id']
        elif ('tenant_id' in resource and
              resource['tenant_id'] != context.tenant_id):
            reason = 'Cannot create resource for another tenant'
            raise n_exc.AdminRequired(reason=reason)
        else:
            tenant_id = context.tenant_id
        return tenant_id

    def _get_resource_name(self, resource, parent, name, uuid):
        """ Generate an unique name. This is display name if there are
        no conflicts or display_name + uuid.
        """
        fq_name = list(parent.fq_name)
        fq_name.append(name)
        try:
            obj = self._api.fq_name_to_id(resource, fq_name)
        except NoIdError:
            return name

        return name + '-' + uuid

    def _is_authorized(self, context, resource):
        return context.is_admin or context.tenant_id == resource['tenant_id']

    def _project_read(self, project_id):
        """ Reads the project from the api server. The project will be
        created it does not yet exist.
        """
        for i in range(self._max_project_read_attempts):
            try:
                return self._api.project_read(id=str(uuid.UUID(project_id)))
            except NoIdError:
                pass
            greenthread.sleep(1)
        raise n_exc.TenantNetworksDisabled()

    def _fields(self, resource, fields):
        if fields:
            return dict(((key, item) for key, item in resource.items()
                         if key in fields))
        return resource

    def _apply_filter(self, resource, filters):
        if filters is None:
            return True
        for key, value in filters.iteritems():
            if key in resource and not resource[key] in value:
                return False
        return True

    def _get_object_status(self, obj):
        id_perms = obj.get_id_perms()
        if id_perms and id_perms.enable:
            return constants.ACTIVE
        return constants.PENDING_DELETE

    def _get_object_description(self, obj):
        id_perms = obj.get_id_perms()
        if id_perms is None:
            return None
        return id_perms.description

    def _get_object_tenant_id(self, obj):
        proj_fq_name = obj.get_fq_name()[0:2]
        try:
            proj = self._api.project_read(fq_name=proj_fq_name)
        except NoIdError:
            return None

        return proj.uuid

    def get_resource(self, context, id, fields=None):
        """ Implement GET by uuid.
        """
        try:
            obj = self.resource_read(id=id)
        except NoIdError:
            raise self.get_exception_notfound(id=id)
        tenant_id = str(uuid.UUID(context.tenant_id))
        project_id = self._get_object_tenant_id(obj)
        if not context.is_admin and tenant_id != project_id:
            raise self.get_exception_notfound(id=id)
        return self.make_dict(obj, fields)

    def _get_resource_dict(self, uuid, filters, fields):
        try:
            obj = self.resource_read(id=uuid)
        except NoIdError:
            return None
        res = self.make_dict(obj, None)
        if not self._apply_filter(res, filters):
            return None
        return self._fields(res, fields)

    def get_collection(self, context, filters=None, fields=None):
        """ Generic implementation of list command.
        """

        response = []

        if filters and 'id' in filters:
            for v in filters['id']:
                res = self._get_resource_dict(v, filters, fields)
                if res is not None and self._is_authorized(context, res):
                    response.append(res)
            return response

        tenant_id = None
        if not context.is_admin:
            tenant_id = context.tenant_id
        obj_list = self.resource_list(tenant_id=tenant_id)

        if self.resource_name_plural not in obj_list:
            return response

        for v in obj_list[self.resource_name_plural]:
            res = self._get_resource_dict(v['uuid'], filters, fields)
            if res is not None:
                response.append(res)
        return response

    def delete(self, context, id):
        if not context.is_admin:
            try:
                obj = self.resource_read(id=id)
            except NoIdError:
                raise self.get_exception_notfound(id=id)
            tenant_id = str(uuid.UUID(context.tenant_id))
            project_id = self._get_object_tenant_id(obj)
            if tenant_id != project_id:
                raise n_exc.NotAuthorized()

        try:
            self.resource_delete(id=id)
        except NoIdError:
            raise self.get_exception_notfound(id=id)
        except RefsExistError:
            raise self.get_exception_inuse(id=id)

    def update_properties_subr(self, props, resource):
        """ Update the DB properties object from the neutron parameters.
        """
        change = False
        for key, mapping in self.property_type_mapping.iteritems():
            if mapping not in resource:
                continue
            if getattr(props, key) != resource[mapping]:
                setattr(props, key, resource[mapping])
                change = True

        return change

    def update(self, context, id, resource):
        """ Update the resource.
        """
        try:
            obj = self.resource_read(id=id)
        except NoIdError:
            raise self.get_exception_notfound(id=id)

        id_perms = obj.get_id_perms()
        if not id_perms or not id_perms.enable:
            raise loadbalancer.StateInvalid(id=id,
                                            state=constants.PENDING_DELETE)
        r = resource[self.neutron_name]
        if r:
            update = False
            if 'description' in r and id_perms.description != r['description']:
                id_perms.description = r['description']
                obj.set_id_perms(id_perms)
                update = True

            if self.update_properties(obj, id, r):
                update = True
            if self.update_object(obj, id, r):
                update = True

            if update:
                self.resource_update(obj)

        return self.make_dict(obj)



#
# Copyright (c) 2014 Juniper Networks, Inc. All rights reserved.
#

import requests
import time
import uuid

from oslo.config import cfg
from cfgm_common import exceptions as vnc_exc
from neutron.common import exceptions as n_exc
from neutron.extensions import loadbalancer
from neutron.extensions.loadbalancer import LoadBalancerPluginBase
from vnc_api.vnc_api import VncApi

import loadbalancer_healthmonitor
import loadbalancer_member
import loadbalancer_pool
import virtual_ip


class LoadBalancerPluginDb(LoadBalancerPluginBase):

    def __init__(self):
        admin_user = cfg.CONF.keystone_authtoken.admin_user
        admin_password = cfg.CONF.keystone_authtoken.admin_password
        admin_tenant_name = cfg.CONF.keystone_authtoken.admin_tenant_name
        api_srvr_ip = cfg.CONF.APISERVER.api_server_ip
        api_srvr_port = cfg.CONF.APISERVER.api_server_port
        try:
            auth_host = cfg.CONF.keystone_authtoken.auth_host
        except cfg.NoSuchOptError:
            auth_host = "127.0.0.1"

        try:
            auth_protocol = cfg.CONF.keystone_authtoken.auth_protocol
        except cfg.NoSuchOptError:
            auth_protocol = "http"

        try:
            auth_port = cfg.CONF.keystone_authtoken.auth_port
        except cfg.NoSuchOptError:
            auth_port = "35357"

        try:
            auth_url = cfg.CONF.keystone_authtoken.auth_url
        except cfg.NoSuchOptError:
            auth_url = "/v2.0/tokens"

        try:
            auth_type = cfg.CONF.keystone_authtoken.auth_type
        except cfg.NoSuchOptError:
            auth_type = "keystone"

        try:
            api_server_url = cfg.CONF.APISERVER.api_server_url
        except cfg.NoSuchOptError:
            api_server_url = "/"

        # Retry till a api-server is up
        connected = False
        while not connected:
            try:
                self._api = VncApi(
                     admin_user, admin_password, admin_tenant_name,
                     api_srvr_ip, api_srvr_port, api_server_url,
                     auth_host=auth_host, auth_port=auth_port,
                     auth_protocol=auth_protocol, auth_url=auth_url,
                     auth_type=auth_type)
                connected = True
            except requests.exceptions.RequestException as e:
                time.sleep(3)

        self._pool_manager = \
            loadbalancer_pool.LoadbalancerPoolManager(self._api)
        self._vip_manager = virtual_ip.VirtualIpManager(self._api)
        self._member_manager = \
            loadbalancer_member.LoadbalancerMemberManager(self._api)
        self._monitor_manager = \
            loadbalancer_healthmonitor.LoadbalancerHealthmonitorManager(
                self._api)

    def get_api_client(self):
        return self._api

    def get_vips(self, context, filters=None, fields=None):
        return self._vip_manager.get_collection(context, filters, fields)

    def get_vip(self, context, id, fields=None):
        return self._vip_manager.get_resource(context, id, fields)

    def create_vip(self, context, vip):
        try:
            return self._vip_manager.create(context, vip)
        except vnc_exc.PermissionDenied as ex:
            raise n_exc.BadRequest(resource='vip', msg=str(ex))

    def update_vip(self, context, id, vip):
        return self._vip_manager.update(context, id, vip)

    def delete_vip(self, context, id):
        return self._vip_manager.delete(context, id)

    def get_pools(self, context, filters=None, fields=None):
        return self._pool_manager.get_collection(context, filters, fields)

    def get_pool(self, context, id, fields=None):
        return self._pool_manager.get_resource(context, id, fields)

    def create_pool(self, context, pool):
        try:
            return self._pool_manager.create(context, pool)
        except vnc_exc.PermissionDenied as ex:
            raise n_exc.BadRequest(resource='pool', msg=str(ex))

    def update_pool(self, context, id, pool):
        return self._pool_manager.update(context, id, pool)

    def delete_pool(self, context, id):
        return self._pool_manager.delete(context, id)

    def stats(self, context, pool_id):
        pass

    def create_pool_health_monitor(self, context, health_monitor, pool_id):
        """ Associate an health monitor with a pool.
        """
        m = health_monitor['health_monitor']
        try:
            pool = self._api.loadbalancer_pool_read(id=pool_id)
        except NoIdError:
            raise loadbalancer.PoolNotFound(pool_id=pool_id)

        try:
            monitor = self._api.loadbalancer_healthmonitor_read(id=m['id'])
        except NoIdError:
            raise loadbalancer.HealthMonitorNotFound(monitor_id=m['id'])

        if not context.is_admin:
            tenant_id = str(uuid.UUID(context.tenant_id))
            if tenant_id != pool.parent_uuid or \
                    tenant_id != monitor.parent_uuid:
                raise n_exc.NotAuthorized()

        pool_refs = monitor.get_loadbalancer_pool_back_refs()
        if pool_refs is not None:
            for ref in pool_refs:
                if ref['uuid'] == pool_id:
                    raise loadbalancer.PoolMonitorAssociationExists(
                        monitor_id=m['id'], pool_id=pool_id)

        pool.add_loadbalancer_healthmonitor(monitor)
        self._api.loadbalancer_pool_update(pool)

        res = {
            'id': monitor.uuid,
            'tenant_id': monitor.parent_uuid.replace('-', '')
        }
        return res

    def get_pool_health_monitor(self, context, id, pool_id, fields=None):
        """ Query a specific pool, health_monitor association.
        """
        try:
            pool = self._api.loadbalancer_pool_read(id=pool_id)
        except NoIdError:
            raise loadbalancer.PoolNotFound(pool_id=id)
        tenant_id = str(uuid.UUID(context.tenant_id))
        if not context.is_admin and tenant_id != pool.parent_uuid:
            raise loadbalancer.PoolNotFound(pool_id=id)

        in_list = False
        for mref in pool.get_loadbalancer_healthmonitor_refs() or []:
            if mref['uuid'] == id:
                in_list = True
                break

        if not in_list:
            raise loadbalancer.PoolMonitorAssociationNotFound(
                monitor_id=id, pool_id=pool_id)

        res = {
            'pool_id': pool_id,
            'monitor_id': id,
            'status': self._pool_manager._get_object_status(pool),
            'tenant_id': pool.parent_uuid.replace('-', '')
        }
        return self._pool_manager._fields(res, fields)

    def delete_pool_health_monitor(self, context, id, pool_id):
        try:
            pool = self._api.loadbalancer_pool_read(id=pool_id)
        except NoIdError:
            raise loadbalancer.PoolNotFound(pool_id=id)
        tenant_id = str(uuid.UUID(context.tenant_id))
        if not context.is_admin and tenant_id != pool.parent_uuid:
            raise loadbalancer.PoolNotFound(pool_id=id)

        try:
            monitor = self._api.loadbalancer_healthmonitor_read(id=id)
        except NoIdError:
            raise loadbalancer.HealthMonitorNotFound(monitor_id=id)

        in_list = False
        for mref in pool.get_loadbalancer_healthmonitor_refs():
            if mref['uuid'] == id:
                in_list = True
                break

        if not in_list:
            raise loadbalancer.PoolMonitorAssociationNotFound(
                monitor_id=id, pool_id=pool_id)

        pool.del_loadbalancer_healthmonitor(monitor)
        self._api.loadbalancer_pool_update(pool)

    def get_members(self, context, filters=None, fields=None):
        return self._member_manager.get_collection(context, filters, fields)

    def get_member(self, context, id, fields=None):
        return self._member_manager.get_resource(context, id, fields)

    def create_member(self, context, member):
        try:
            return self._member_manager.create(context, member)
        except vnc_exc.PermissionDenied as ex:
            raise n_exc.BadRequest(resource='member', msg=str(ex))

    def update_member(self, context, id, member):
        return self._member_manager.update(context, id, member)

    def delete_member(self, context, id):
        return self._member_manager.delete(context, id)

    def get_health_monitors(self, context, filters=None, fields=None):
        return self._monitor_manager.get_collection(context, filters, fields)

    def get_health_monitor(self, context, id, fields=None):
        return self._monitor_manager.get_resource(context, id, fields)

    def create_health_monitor(self, context, health_monitor):
        try:
            return self._monitor_manager.create(context, health_monitor)
        except vnc_exc.PermissionDenied as ex:
            raise n_exc.BadRequest(resource='health_monitor', msg=str(ex))

    def update_health_monitor(self, context, id, health_monitor):
        return self._monitor_manager.update(context, id, health_monitor)

    def delete_health_monitor(self, context, id):
        return self._monitor_manager.delete(context, id)



# Copyright 2014 Juniper Networks.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Hampapur Ajay, Praneet Bachheti, Rudra Rugge, Atul Moghe

from oslo.config import cfg
import requests

from neutron.api.v2 import attributes as attr
from neutron.common import exceptions as exc
from neutron.db import portbindings_base
from neutron.db import quota_db  # noqa
from neutron.extensions import allowedaddresspairs
from neutron.extensions import external_net
from neutron.extensions import l3
from neutron.extensions import portbindings
from neutron.extensions import securitygroup
from neutron import neutron_plugin_base_v2
try:
    from neutron.openstack.common import importutils
except ImportError:
    from oslo_utils import importutils

try:
    from neutron.openstack.common import jsonutils as json
except ImportError:
    from oslo_serialization import jsonutils as json

from neutron.openstack.common import log as logging
from simplejson import JSONDecodeError

LOG = logging.getLogger(__name__)

vnc_opts = [
    cfg.StrOpt('api_server_ip', default='127.0.0.1',
               help='IP address to connect to VNC controller'),
    cfg.StrOpt('api_server_port', default='8082',
               help='Port to connect to VNC controller'),
    cfg.DictOpt('contrail_extensions', default={},
                help='Enable Contrail extensions(policy, ipam)'),
]

class InvalidContrailExtensionError(exc.ServiceUnavailable):
    message = _("Invalid Contrail Extension: %(ext_name) %(ext_class)")

class NeutronPluginContrailCoreV2(neutron_plugin_base_v2.NeutronPluginBaseV2,
                                  securitygroup.SecurityGroupPluginBase,
                                  portbindings_base.PortBindingBaseMixin,
                                  external_net.External_net):

    supported_extension_aliases = ["security-group", "router",
                                   "port-security", "binding", "agent",
                                   "quotas", "external-net", "contrail",
                                   "allowed-address-pairs", "extra_dhcp_opt"]
    PLUGIN_URL_PREFIX = '/neutron'
    __native_bulk_support = False

    # patch VIF_TYPES
    portbindings.__dict__['VIF_TYPE_VROUTER'] = 'vrouter'
    portbindings.VIF_TYPES.append(portbindings.VIF_TYPE_VROUTER)

    def _parse_class_args(self):
        """Parse the contrailplugin.ini file.

        Opencontrail supports extension such as ipam, policy, these extensions
        can be configured in the plugin configuration file as shown below.
        Plugin then loads the specified extensions.
        contrail_extensions=ipam:<classpath>,policy:<classpath>
        """

        contrail_extensions = cfg.CONF.APISERVER.contrail_extensions
        # If multiple class specified for same extension, last one will win
        # according to DictOpt behavior
        for ext_name, ext_class in contrail_extensions.items():
            try:
                if not ext_class:
                    LOG.error(_('Malformed contrail extension...'))
                    continue
                self.supported_extension_aliases.append(ext_name)
                if ext_class == 'None':
                    continue
                ext_class = importutils.import_class(ext_class)
                ext_instance = ext_class()
                ext_instance.set_core(self)
                for method in dir(ext_instance):
                    for prefix in ['get', 'update', 'delete', 'create']:
                        if method.startswith('%s_' % prefix):
                            setattr(self, method,
                                    ext_instance.__getattribute__(method))
            except Exception:
                LOG.exception(_("Contrail Backend Error"))
                # Converting contrail backend error to Neutron Exception
                raise InvalidContrailExtensionError(
                    ext_name=ext_name, ext_class=ext_class)

        #keystone
        self._authn_token = None
        if cfg.CONF.auth_strategy == 'keystone':
            kcfg = cfg.CONF.keystone_authtoken
            body = '{"auth":{"passwordCredentials":{'
            body += ' "username": "%s",' % (kcfg.admin_user)
            body += ' "password": "%s"},' % (kcfg.admin_password)
            body += ' "tenantName":"%s"}}' % (kcfg.admin_tenant_name)

            self._authn_body = body
            self._authn_token = cfg.CONF.keystone_authtoken.admin_token
            self._keystone_url = "%s://%s:%s%s" % (
                cfg.CONF.keystone_authtoken.auth_protocol,
                cfg.CONF.keystone_authtoken.auth_host,
                cfg.CONF.keystone_authtoken.auth_port,
                "/v2.0/tokens")

    def __init__(self):
        super(NeutronPluginContrailCoreV2, self).__init__()
        portbindings_base.register_port_dict_function()
        cfg.CONF.register_opts(vnc_opts, 'APISERVER')
        self._parse_class_args()
        self.base_binding_dict = self._get_base_binding_dict()

    def _get_base_binding_dict(self):
        binding = {
            portbindings.VIF_TYPE: portbindings.VIF_TYPE_VROUTER,
        }
        # In havana portbindings.VIF_DETAILS is not defined by neutron
        try:
            binding[portbindings.VIF_DETAILS] = {
                # TODO(praneetb): Replace with new VIF security details
                portbindings.CAP_PORT_FILTER:
                'security-group' in self.supported_extension_aliases
            }
        except AttributeError:
            pass
        return binding

    def get_agents(self, context, filters=None, fields=None):
        # This method is implemented so that horizon is happy
        return []

    def _request_api_server(self, url, data=None, headers=None):
        # Attempt to post to Api-Server
        response = requests.post(url, data=data, headers=headers)
        if (response.status_code == requests.codes.unauthorized):
            # Get token from keystone and save it for next request
            response = requests.post(self._keystone_url,
                data=self._authn_body,
                headers={'Content-type': 'application/json'})
            if (response.status_code == requests.codes.ok):
                # plan is to re-issue original request with new token
                auth_headers = headers or {}
                authn_content = json.loads(response.text)
                self._authn_token = authn_content['access']['token']['id']
                auth_headers['X-AUTH-TOKEN'] = self._authn_token
                response = self._request_api_server(url, data, auth_headers)
            else:
                raise RuntimeError('Authentication Failure')
        return response

    def _request_api_server_authn(self, url, data=None, headers=None):
        authn_headers = headers or {}
        if self._authn_token is not None:
            authn_headers['X-AUTH-TOKEN'] = self._authn_token
        response = self._request_api_server(url, data, headers=authn_headers)
        return response

    def _relay_request(self, url_path, data=None):
        """Send received request to api server."""

        url = "http://%s:%s%s" % (cfg.CONF.APISERVER.api_server_ip,
                                  cfg.CONF.APISERVER.api_server_port,
                                  url_path)

        return self._request_api_server_authn(
            url, data=data, headers={'Content-type': 'application/json'})

    def _request_backend(self, context, data_dict, obj_name, action):
        context_dict = self._encode_context(context, action, obj_name)
        data = json.dumps({'context': context_dict, 'data': data_dict})

        url_path = "%s/%s" % (self.PLUGIN_URL_PREFIX, obj_name)
        response = self._relay_request(url_path, data=data)
        try:
            return response.status_code, response.json()
        except JSONDecodeError:
            return response.status_code, {'message': response.content}

    def _encode_context(self, context, operation, apitype):
        cdict = {'user_id': getattr(context, 'user_id', ''),
                 'is_admin': getattr(context, 'is_admin', False),
                 'operation': operation,
                 'type': apitype,
                 'tenant_id': getattr(context, 'tenant_id', None)}
        if context.roles:
            cdict['roles'] = context.roles
        if context.tenant:
            cdict['tenant'] = context.tenant
        return cdict

    def _encode_resource(self, resource_id=None, resource=None, fields=None,
                         filters=None):
        resource_dict = {}
        if resource_id:
            resource_dict['id'] = resource_id
        if resource:
            resource_dict['resource'] = resource
        resource_dict['filters'] = filters
        resource_dict['fields'] = fields
        return resource_dict

    def _prune(self, resource_dict, fields):
        if fields:
            return dict(((key, item) for key, item in resource_dict.items()
                         if key in fields))
        return resource_dict

    def _transform_response(self, status_code, info=None, obj_name=None,
                            fields=None):
        if status_code == requests.codes.ok:
            if not isinstance(info, list):
                return self._prune(info, fields)
            else:
                return [self._prune(items, fields) for items in info]
        self._raise_contrail_error(info, obj_name)

    def _raise_contrail_error(self, info, obj_name):
        exc_name = info.get('exception')
        if exc_name:
            if exc_name == 'BadRequest' and 'resource' not in info:
                info['resource'] = obj_name
            if hasattr(exc, exc_name):
                raise getattr(exc, exc_name)(**info)
            if hasattr(l3, exc_name):
                raise getattr(l3, exc_name)(**info)
            if hasattr(securitygroup, exc_name):
                raise getattr(securitygroup, exc_name)(**info)
            if hasattr(allowedaddresspairs, exc_name):
                raise getattr(allowedaddresspairs, exc_name)(**info)
        raise exc.NeutronException(**info)

    def _create_resource(self, res_type, context, res_data):
        """Create a resource in API server.

        This method encodes neutron model, and sends it to the
        contrail api server.
        """

        for key, value in res_data[res_type].items():
            if value == attr.ATTR_NOT_SPECIFIED:
                del res_data[res_type][key]

        res_dict = self._encode_resource(resource=res_data[res_type])
        status_code, res_info = self._request_backend(context, res_dict,
                                                      res_type, 'CREATE')
        res_dicts = self._transform_response(status_code, info=res_info,
                                             obj_name=res_type)
        LOG.debug("create_%(res_type)s(): %(res_dicts)s",
                  {'res_type': res_type, 'res_dicts': res_dicts})

        return res_dicts

    def _get_resource(self, res_type, context, id, fields):
        """Get a resource from API server.

        This method gets a resource from the contrail api server
        """

        res_dict = self._encode_resource(resource_id=id, fields=fields)
        status_code, res_info = self._request_backend(context, res_dict,
                                                      res_type, 'READ')
        res_dicts = self._transform_response(status_code, info=res_info,
                                             fields=fields, obj_name=res_type)
        LOG.debug("get_%(res_type)s(): %(res_dicts)s",
                  {'res_type': res_type, 'res_dicts': res_dicts})

        return res_dicts

    def _update_resource(self, res_type, context, id, res_data):
        """Update a resource in API server.

        This method updates a resource in the contrail api server
        """

        res_dict = self._encode_resource(resource_id=id,
                                         resource=res_data[res_type])
        status_code, res_info = self._request_backend(context, res_dict,
                                                      res_type, 'UPDATE')
        res_dicts = self._transform_response(status_code, info=res_info,
                                             obj_name=res_type)
        LOG.debug("update_%(res_type)s(): %(res_dicts)s",
                  {'res_type': res_type, 'res_dicts': res_dicts})

        return res_dicts

    def _delete_resource(self, res_type, context, id):
        """Delete a resource in API server

        This method deletes a resource in the contrail api server
        """

        res_dict = self._encode_resource(resource_id=id)
        LOG.debug("delete_%(res_type)s(): %(id)s",
                  {'res_type': res_type, 'id': id})
        status_code, res_info = self._request_backend(context, res_dict,
                                                      res_type, 'DELETE')
        if status_code != requests.codes.ok:
            self._raise_contrail_error(info=res_info,
                                       obj_name=res_type)

    def _list_resource(self, res_type, context, filters, fields):
        res_dict = self._encode_resource(filters=filters, fields=fields)
        status_code, res_info = self._request_backend(context, res_dict,
                                                      res_type, 'READALL')
        res_dicts = self._transform_response(status_code, info=res_info,
                                             fields=fields, obj_name=res_type)
        LOG.debug(
            "get_%(res_type)s(): filters: %(filters)r data: %(res_dicts)r",
            {'res_type': res_type, 'filters': filters,
             'res_dicts': res_dicts})

        return res_dicts

    def _count_resource(self, res_type, context, filters):
        res_dict = self._encode_resource(filters=filters)
        status_code, res_count = self._request_backend(context, res_dict,
                                                       res_type, 'READCOUNT')
        LOG.debug("get_%(res_type)s_count(): %(res_count)r",
                  {'res_type': res_type, 'res_count': res_count})
        return res_count

    def _get_network(self, context, id, fields=None):
        return self._get_resource('network', context, id, fields)

    def create_network(self, context, network):
        """Creates a new Virtual Network."""

        return self._create_resource('network', context, network)

    def get_network(self, context, network_id, fields=None):
        """Get the attributes of a particular Virtual Network."""

        return self._get_network(context, network_id, fields)

    def update_network(self, context, network_id, network):
        """Updates the attributes of a particular Virtual Network."""

        return self._update_resource('network', context, network_id,
                                     network)

    def delete_network(self, context, network_id):
        """Creates a new Virtual Network.

        Deletes the network with the specified network identifier
        belonging to the specified tenant.
        """

        self._delete_resource('network', context, network_id)

    def get_networks(self, context, filters=None, fields=None):
        """Get the list of Virtual Networks."""

        return self._list_resource('network', context, filters,
                                   fields)

    def get_networks_count(self, context, filters=None):
        """Get the count of Virtual Network."""

        networks_count = self._count_resource('network', context, filters)
        return networks_count['count']

    def create_subnet(self, context, subnet):
        """Creates a new subnet, and assigns it a symbolic name."""

        if subnet['subnet']['gateway_ip'] is None:
            subnet['subnet']['gateway_ip'] = '0.0.0.0'

        if subnet['subnet']['host_routes'] != attr.ATTR_NOT_SPECIFIED:
            if (len(subnet['subnet']['host_routes']) >
                    cfg.CONF.max_subnet_host_routes):
                raise exc.HostRoutesExhausted(subnet_id=subnet[
                    'subnet'].get('id', _('new subnet')),
                    quota=cfg.CONF.max_subnet_host_routes)

        subnet_created = self._create_resource('subnet', context, subnet)
        return self._make_subnet_dict(subnet_created)

    def _make_subnet_dict(self, subnet):
        return subnet

    def _get_subnet(self, context, subnet_id, fields=None):
        subnet = self._get_resource('subnet', context, subnet_id, fields)
        return self._make_subnet_dict(subnet)

    def get_subnet(self, context, subnet_id, fields=None):
        """Get the attributes of a particular subnet."""

        return self._get_subnet(context, subnet_id, fields)

    def update_subnet(self, context, subnet_id, subnet):
        """Updates the attributes of a particular subnet."""

        subnet = self._update_resource('subnet', context, subnet_id, subnet)
        return self._make_subnet_dict(subnet)

    def delete_subnet(self, context, subnet_id):
        """
        Deletes the subnet with the specified subnet identifier
        belonging to the specified tenant.
        """

        self._delete_resource('subnet', context, subnet_id)

    def get_subnets(self, context, filters=None, fields=None):
        """Get the list of subnets."""

        return [self._make_subnet_dict(s)
                for s in self._list_resource(
                    'subnet', context, filters, fields)]

    def get_subnets_count(self, context, filters=None):
        """Get the count of subnets."""

        subnets_count = self._count_resource('subnet', context, filters)
        return subnets_count['count']

    def _extend_port_dict_security_group(self, port_res, port_db):
        # Security group bindings will be retrieved from the sqlalchemy
        # model. As they're loaded eagerly with ports because of the
        # joined load they will not cause an extra query.
        port_res[securitygroup.SECURITYGROUPS] = port_db.get(
            'security_groups', []) or []
        return port_res

    def _make_port_dict(self, port, fields=None):
        """filters attributes of a port based on fields."""

        if not fields:
            port.update(self.base_binding_dict)
        else:
            for key in self.base_binding_dict:
                if key in fields:
                    port.update(self.base_binding_dict[key])
        return port

    def _get_port(self, context, id, fields=None):
        port = self._get_resource('port', context, id, fields)
        return self._make_port_dict(port)

    def _update_ips_for_port(self, context, network_id, port_id, original_ips,
                             new_ips):
        """Add or remove IPs from the port."""

        # These ips are still on the port and haven't been removed
        prev_ips = []

        # the new_ips contain all of the fixed_ips that are to be updated
        if len(new_ips) > cfg.CONF.max_fixed_ips_per_port:
            msg = _('Exceeded maximim amount of fixed ips per port')
            raise exc.InvalidInput(error_message=msg)

        # Remove all of the intersecting elements
        for original_ip in original_ips[:]:
            for new_ip in new_ips[:]:
                if ('ip_address' in new_ip and
                        original_ip['ip_address'] == new_ip['ip_address']):
                    original_ips.remove(original_ip)
                    new_ips.remove(new_ip)
                    prev_ips.append(original_ip)

        return new_ips, prev_ips

    def create_port(self, context, port):
        """Creates a port on the specified Virtual Network."""

        port = self._create_resource('port', context, port)
        return self._make_port_dict(port)

    def get_port(self, context, port_id, fields=None):
        """Get the attributes of a particular port."""

        return self._get_port(context, port_id, fields)

    def update_port(self, context, port_id, port):
        """Updates a port.

        Updates the attributes of a port on the specified Virtual
        Network.
        """

        if 'fixed_ips' in port['port']:
            original = self._get_port(context, port_id)
            added_ips, prev_ips = self._update_ips_for_port(
                context, original['network_id'], port_id,
                original['fixed_ips'], port['port']['fixed_ips'])
            port['port']['fixed_ips'] = prev_ips + added_ips

        port = self._update_resource('port', context, port_id, port)
        return self._make_port_dict(port)

    def delete_port(self, context, port_id):
        """Deletes a port.

        Deletes a port on a specified Virtual Network,
        if the port contains a remote interface attachment,
        the remote interface is first un-plugged and then the port
        is deleted.
        """

        self._delete_resource('port', context, port_id)

    def get_ports(self, context, filters=None, fields=None):
        """Get all ports.

        Retrieves all port identifiers belonging to the
        specified Virtual Network with the specfied filter.
        """

        return [self._make_port_dict(p)
                for p in self._list_resource('port', context, filters, fields)]

    def get_ports_count(self, context, filters=None):
        """Get the count of ports."""

        ports_count = self._count_resource('port', context, filters)
        return ports_count['count']

    # Router API handlers
    def create_router(self, context, router):
        """Creates a router.

        Creates a new Logical Router, and assigns it
        a symbolic name.
        """

        return self._create_resource('router', context, router)

    def get_router(self, context, router_id, fields=None):
        """Get the attributes of a router."""

        return self._get_resource('router', context, router_id, fields)

    def update_router(self, context, router_id, router):
        """Updates the attributes of a router."""

        return self._update_resource('router', context, router_id,
                                     router)

    def delete_router(self, context, router_id):
        """Deletes a router."""

        self._delete_resource('router', context, router_id)

    def get_routers(self, context, filters=None, fields=None):
        """Retrieves all router identifiers."""

        return self._list_resource('router', context, filters, fields)

    def get_routers_count(self, context, filters=None):
        """Get the count of routers."""

        routers_count = self._count_resource('router', context, filters)
        return routers_count['count']

    def add_router_interface(self, context, router_id, interface_info):
        """Add interface to a router."""

        if not interface_info:
            msg = _("Either subnet_id or port_id must be specified")
            raise exc.BadRequest(resource='router', msg=msg)

        if 'port_id' in interface_info:
            if 'subnet_id' in interface_info:
                msg = _("Cannot specify both subnet-id and port-id")
                raise exc.BadRequest(resource='router', msg=msg)

        res_dict = self._encode_resource(resource_id=router_id,
                                         resource=interface_info)
        status_code, res_info = self._request_backend(context, res_dict,
                                                      'router', 'ADDINTERFACE')
        if status_code != requests.codes.ok:
            self._raise_contrail_error(info=res_info,
                                       obj_name='add_router_interface')
        return res_info

    def remove_router_interface(self, context, router_id, interface_info):
        """Delete interface from a router."""

        if not interface_info:
            msg = _("Either subnet_id or port_id must be specified")
            raise exc.BadRequest(resource='router', msg=msg)

        res_dict = self._encode_resource(resource_id=router_id,
                                         resource=interface_info)
        status_code, res_info = self._request_backend(context, res_dict,
                                                      'router', 'DELINTERFACE')
        if status_code != requests.codes.ok:
            self._raise_contrail_error(info=res_info,
                                       obj_name='remove_router_interface')
        return res_info

    # Floating IP API handlers
    def create_floatingip(self, context, floatingip):
        """Creates a floating IP."""

        return self._create_resource('floatingip', context, floatingip)

    def update_floatingip(self, context, fip_id, floatingip):
        """Updates the attributes of a floating IP."""

        return self._update_resource('floatingip', context, fip_id,
                                     floatingip)

    def get_floatingip(self, context, fip_id, fields=None):
        """Get the attributes of a floating ip."""

        return self._get_resource('floatingip', context, fip_id, fields)

    def delete_floatingip(self, context, fip_id):
        """Deletes a floating IP."""

        self._delete_resource('floatingip', context, fip_id)

    def get_floatingips(self, context, filters=None, fields=None):
        """Retrieves all floating ips identifiers."""

        return self._list_resource('floatingip', context, filters, fields)

    def get_floatingips_count(self, context, filters=None):
        """Get the count of floating IPs."""

        fips_count = self._count_resource('floatingip', context, filters)
        return fips_count['count']

    # Security Group handlers
    def create_security_group(self, context, security_group):
        """Creates a Security Group."""

        return self._create_resource('security_group', context,
                                     security_group)

    def get_security_group(self, context, sg_id, fields=None, tenant_id=None):
        """Get the attributes of a security group."""

        return self._get_resource('security_group', context, sg_id, fields)

    def update_security_group(self, context, sg_id, security_group):
        """Updates the attributes of a security group."""

        return self._update_resource('security_group', context, sg_id,
                                     security_group)

    def delete_security_group(self, context, sg_id):
        """Deletes a security group."""

        self._delete_resource('security_group', context, sg_id)

    def get_security_groups(self, context, filters=None, fields=None,
                            sorts=None, limit=None, marker=None,
                            page_reverse=False):
        """Retrieves all security group identifiers."""

        return self._list_resource('security_group', context,
                                   filters, fields)

    def get_security_groups_count(self, context, filters=None):
        return 0

    def get_security_group_rules_count(self, context, filters=None):
        return 0

    def create_security_group_rule(self, context, security_group_rule):
        """Creates a security group rule."""

        return self._create_resource('security_group_rule', context,
                                     security_group_rule)

    def delete_security_group_rule(self, context, sg_rule_id):
        """Deletes a security group rule."""

        self._delete_resource('security_group_rule', context, sg_rule_id)

    def get_security_group_rule(self, context, sg_rule_id, fields=None):
        """Get the attributes of a security group rule."""

        return self._get_resource('security_group_rule', context,
                                  sg_rule_id, fields)

    def get_security_group_rules(self, context, filters=None, fields=None,
                                 sorts=None, limit=None, marker=None,
                                 page_reverse=False):
        """Retrieves all security group rules."""

        return self._list_resource('security_group_rule', context,
                                   filters, fields)



import mock
import unittest
import uuid

from neutron_plugin_contrail.plugins.opencontrail.quota.driver import QuotaDriver

class ContrailPluginQuotaDriverTest(unittest.TestCase):
    def setUp(self):
        print "setup quota"

    def test_testenv(self):
        print "testenv quota ok"

    def test_get_tenant_quotas_arg(self):
        """Call neutron_plugin_contrail.plugins.opencontrail.quota.driver.QuotaDriver.[ _get_quotas, get_all_quotas ]"""

        class MockContext():
            tenant_id = 'f00dbeef012f411b89d68928ee8703ee'

        class MockResource():
            name = 'default'
            default = -1
            def __init__(self, name = 'default', default = -1):
                self.name = name
                self.default = default
            
        driver = QuotaDriver()
        ctx = MockContext()

        foo_quotas = {'network': 5}
        default_quotas = {'network': MockResource('network', 5) }
        target_tenant = 'f00dbeef012f411b89d68928ee8703ee'

        with mock.patch.object(QuotaDriver,
                               'get_tenant_quotas',
                               return_value=foo_quotas) as get_tenant_quotas:

            quotas = driver._get_quotas(ctx,
                                        default_quotas,
                                        ['network'])
            self.assertEqual(quotas, foo_quotas)

            quotas = driver.get_all_quotas(ctx,
                                           default_quotas)
            self.assertEqual(quotas[0], foo_quotas)
            get_tenant_quotas.assert_called_once_with(ctx,
                                                      default_quotas,
                                                      target_tenant)



from neutron.common import exceptions as n_exc


def get_subnet_network_id(client, subnet_id):
    try:
        kv_pair = client.kv_retrieve(subnet_id)
    except NoIdError:
        raise n_exc.SubnetNotFound(subnet_id=subnet_id)
    return kv_pair.split()[0]

def get_subnet_cidr(client, subnet_id):
    try:
        kv_pair = client.kv_retrieve(subnet_id)
    except NoIdError:
        raise n_exc.SubnetNotFound(subnet_id=subnet_id)
    return kv_pair.split()[1]



#
# Copyright (c) 2014 Juniper Networks, Inc. All rights reserved.
#

import mock
import unittest
import uuid

from neutron.openstack.common import uuidutils

from vnc_api.vnc_api import InstanceIp
from vnc_api.vnc_api import LoadbalancerPool, LoadbalancerPoolType
from vnc_api.vnc_api import Project
from vnc_api.vnc_api import ServiceInstance, ServiceInstanceType
from vnc_api.vnc_api import ServiceTemplate
from vnc_api.vnc_api import VirtualIp, VirtualIpType
from vnc_api.vnc_api import VirtualMachineInterface
from vnc_api.vnc_api import VirtualNetwork
from vnc_api.vnc_api import NoIdError


import importlib
_MODULE_PATH = 'neutron_plugin_contrail.plugins.opencontrail.loadbalancer'
mod_driver = importlib.import_module('%s.driver' % _MODULE_PATH)
mod_plugin = importlib.import_module('%s.plugin' % _MODULE_PATH)

_uuid = uuidutils.generate_uuid


class LoadbalancerDriverTest(unittest.TestCase):

    def setUp(self):
        super(LoadbalancerDriverTest, self).setUp()
        self._patcher = mock.patch('vnc_api.vnc_api.VncApi', autospec=True)
        self.api_server = self._patcher.start()
        plugin = mock.Mock(mod_plugin.LoadBalancerPlugin, autospec=True)
        plugin.get_api_client.return_value = self.api_server
        self.driver = mod_driver.OpencontrailLoadbalancerDriver(plugin)

    def tearDown(self):
        self._patcher.stop()
        super(LoadbalancerDriverTest, self).tearDown()

    def test_create_pool(self):
        tenant_id = _uuid()
        pool_id = _uuid()
        vip_id = _uuid()
        subnet_id = _uuid()

        api = self.api_server

        project = Project(name='test')
        project.uuid = tenant_id
        template = ServiceTemplate('lb-test', project)
        template.uuid = _uuid()

        pool_attrs = LoadbalancerPoolType()
        pool_attrs.subnet_id = subnet_id
        pool = LoadbalancerPool(
            pool_id, project, loadbalancer_pool_properties=pool_attrs)
        pool.uuid = pool_id

        vip_attrs = VirtualIpType()
        vip_attrs.subnet_id = subnet_id
        vip_attrs.address = '127.0.0.1'
        vip = VirtualIp(vip_id, project, virtual_ip_properties=vip_attrs)
        vip.uuid = vip_id
        vip.set_loadbalancer_pool(pool)

        vnet = VirtualNetwork('test', project)
        vnet.uuid = _uuid()

        vmi = VirtualMachineInterface(vip_id, project)
        vmi.uuid = _uuid()
        vmi.set_virtual_network(vnet)
        iip = InstanceIp(vip_id, instance_ip_address='127.0.0.1')
        iip.uuid = _uuid()
        iip.set_virtual_machine_interface(vmi)

        iip_refs = [
            {'to': iip.get_fq_name(), 'uuid': iip.uuid}
        ]
        vmi.get_instance_ip_back_refs = mock.Mock()
        vmi.get_instance_ip_back_refs.return_value = iip_refs

        vip.set_virtual_machine_interface(vmi)

        api.virtual_service_template_read = template
        api.loadbalancer_pool_read.return_value = pool
        api.virtual_ip_read.return_value = vip
        api.kv_retrieve.return_value = '%s %s' % (vnet.uuid, subnet_id)
        api.virtual_machine_interface_read.return_value = vmi
        api.instance_ip_read.return_value = iip
        api.service_instance_read.side_effect = NoIdError('404')

        context = {}
        pool_data = {
            'id': pool_id,
            'vip_id': vip_id
        }
        self.driver.create_pool(context, pool_data)
        api.service_instance_create.assert_called_with(mock.ANY)




EXTENDED_ATTRIBUTES_2_0 = {
    'networks': {
        'contrail:fq_name': {'allow_post': False,
                             'allow_put': False,
                             'is_visible': True},
        'contrail:instance_count': {'allow_post': False,
                                    'allow_put': False,
                                    'is_visible': True},
        'contrail:policys': {'allow_post': True,
                             'allow_put': True,
                             'default': '',
                             'is_visible': True},
        'contrail:subnet_ipam': {'allow_post': False,
                                 'allow_put': False,
                                 'default': '',
                                 'is_visible': True},
    },
    'routers': {
        'contrail:fq_name': {'allow_post': False,
                             'allow_put': False,
                             'is_visible': True},
    },
    'security_groups': {
        'contrail:fq_name': {'allow_post': False,
                             'allow_put': False,
                             'is_visible': True},
    },
    'subnets': {
        'contrail:instance_count': {'allow_post': False,
                                    'allow_put': False,
                                    'is_visible': True},
        'contrail:ipam_fq_name': {'allow_post': True,
                                  'allow_put': True,
                                  'default': '',
                                  'is_visible': True},
    }
}


class Contrail(object):

    @classmethod
    def get_name(cls):
        return "Contrail Extension"

    @classmethod
    def get_alias(cls):
        return "contrail"

    @classmethod
    def get_description(cls):
        return ("Contrail Extension")

    @classmethod
    def get_namespace(cls):
        return "http://docs.openstack.org/TODO"

    @classmethod
    def get_updated(cls):
        return "2014-08-12T10:00:00-00:00"

    @classmethod
    def get_resources(cls):
        """ Returns Ext Resources """
        exts = []
        return exts

    def get_extended_resources(self, version):
        if version == "2.0":
            return EXTENDED_ATTRIBUTES_2_0
        else:
            return {}
# end class Contrail


class ContrailPluginBase(object):
    pass
# end class ContrailPluginBase



#
# Copyright (c) 2014 Juniper Networks, Inc. All rights reserved.
#

from neutron.extensions import loadbalancer
from neutron.openstack.common import uuidutils
from neutron.plugins.common import constants
from neutron.tests.unit import test_api_v2
from neutron.tests.unit import test_api_v2_extension
from neutron_plugin_contrail.plugins.opencontrail.loadbalancer.plugin \
    import LoadBalancerPlugin
from vnc_api.vnc_api import IdPermsType
from vnc_api.vnc_api import LoadbalancerHealthmonitor
from vnc_api.vnc_api import LoadbalancerHealthmonitorType
from vnc_api.vnc_api import LoadbalancerMember, LoadbalancerMemberType
from vnc_api.vnc_api import LoadbalancerPool, LoadbalancerPoolType
from vnc_api.vnc_api import Project
from vnc_api.vnc_api import VirtualIp, VirtualIpType
from vnc_api.vnc_api import VirtualNetwork
from vnc_api.vnc_api import VncApi
from webob import exc
import copy
import mock
import neutron.services.loadbalancer.drivers.abstract_driver as abstract_driver

_PLUGIN = 'neutron_plugin_contrail.plugins.opencontrail.' \
    'loadbalancer.plugin.LoadBalancerPlugin'

_uuid = uuidutils.generate_uuid
_get_path = test_api_v2._get_path


class OpencontrailLoadbalancerTest(test_api_v2_extension.ExtensionTestCase):
    fmt = 'json'

    def setUp(self):
        super(OpencontrailLoadbalancerTest, self).setUp()
        self._setUpExtension(
            _PLUGIN,
            constants.LOADBALANCER, loadbalancer.RESOURCE_ATTRIBUTE_MAP,
            loadbalancer.Loadbalancer, 'lb', use_quota=True)

        self._plugin_patcher.stop()

        plugin_path = '.'.join(_PLUGIN.split('.')[:-2])
        db_class_name = plugin_path + '.loadbalancer_db'
        self._vnc_patcher = mock.patch('%s.VncApi' % db_class_name,
                                       autospec=True)
        self._vnc_patcher.start()

        plugin_class_name = plugin_path + '.plugin'
        self._driver_patcher = mock.patch(
            '%s.service_base' % plugin_class_name, autospec=True)
        self._driver = mock.Mock(abstract_driver.LoadBalancerAbstractDriver)
        service_base = self._driver_patcher.start()
        service_base.load_drivers.return_value = ({
            'lbaas': self._driver
        }, 'lbaas')

        self.loadbalancer = LoadBalancerPlugin()
        self.loadbalancer._get_driver_for_pool = self._driver
        self.api_server = self.loadbalancer._api

        self._project = None

    def tearDown(self):
        self._vnc_patcher.stop()
        self._driver_patcher.stop()
        super(OpencontrailLoadbalancerTest, self).tearDown()

    def _project_read(self, *args, **kwargs):
        """ Return a mock project with the expected values """
        project = Project()
        project.uuid = kwargs['id']
        project.fq_name = ['default-domain', 'test']
        self._project = project
        return project

    def _loadbalancer_pool_read(self, *args, **kwargs):
        pool = LoadbalancerPool()
        pool.uuid = kwargs['id']
        pool.parent_uuid = self._project.uuid
        return pool

    def test_vip_create(self):
        def create_vip(*args, **kwargs):
            return self.loadbalancer.create_vip(*args, **kwargs)

        self.plugin.return_value.create_vip.side_effect = create_vip

        self._port_id = None

        def api_virtual_machine_interface_create(*args, **kwargs):
            vmi = args[0]
            vmi.uuid = _uuid()
            self._port_id = vmi.uuid
            return vmi.uuid

        def api_virtual_ip_create(*args, **kwargs):
            vip = args[0]
            vip.parent_uuid = self._project.uuid
            return vip.uuid

        instance = self.api_server
        instance.fq_name_to_id.return_value = None
        instance.project_read.side_effect = self._project_read
        instance.loadbalancer_pool_read.side_effect = \
            self._loadbalancer_pool_read
        instance.kv_retrieve.return_value = '%s %s' % (_uuid(), _uuid())
        vnet = VirtualNetwork('default')
        instance.virtual_network_read.return_value = vnet
        instance.virtual_machine_interface_create.side_effect = \
            api_virtual_machine_interface_create
        instance.virtual_ip_create.side_effect = api_virtual_ip_create

        data = {'vip': {'name': 'vip1',
                        'description': 'descr_vip1',
                        'subnet_id': _uuid(),
                        'address': '127.0.0.1',
                        'protocol_port': 80,
                        'protocol': 'HTTP',
                        'pool_id': _uuid(),
                        'session_persistence': {'type': 'HTTP_COOKIE'},
                        'connection_limit': 100,
                        'admin_state_up': True,
                        'tenant_id': _uuid()}}

        res = self.api.post(_get_path('lb/vips', fmt=self.fmt),
                            self.serialize(data),
                            content_type='application/%s' % self.fmt)

        self.assertEqual(res.status_int, exc.HTTPCreated.code)
        res = self.deserialize(res)
        self.assertIn('vip', res)
        expected = copy.copy(data['vip'])
        expected.update({'status': "ACTIVE",
                         'port_id': self._port_id,
                         'id': res['vip']['id']
                         })
        self.assertEqual(expected, res['vip'])

    def _virtual_ip_properties_build(self):
        props = VirtualIpType()
        props.address = '127.0.0.1'
        props.protocol = 'HTTP'
        props.protocol_port = 80
        props.connection_limit = 100
        props.subnet_id = _uuid()
        props.admin_state = True
        return props

    def _loadbalancer_pool_properties_build(self):
        props = LoadbalancerPoolType()
        props.protocol = 'HTTP'
        props.loadbalancer_method = 'ROUND_ROBIN'
        props.subnet_id = _uuid()
        props.admin_state = True
        return props

    def _loadbalancer_member_properties_build(self):
        props = LoadbalancerMemberType()
        props.address = '127.0.0.1'
        props.protocol_port = 80
        props.weight = 1
        props.admin_state = True
        return props

    def _loadbalancer_healthmonitor_properties_build(self):
        props = LoadbalancerHealthmonitorType()
        props.type = 'HTTP'
        props.delay = 2
        props.timeout = 1
        props.max_retries = 3
        props.http_method = 'GET'
        props.url_path = '/path'
        props.expected_codes = '200-300'
        props.admin_state = True
        return props

    def _create_pool(self, pool_id, tenant_id=None):
        if tenant_id is None:
            tenant_id = _uuid()
        id_perms = IdPermsType(uuid=pool_id, enable=True)
        props = self._loadbalancer_pool_properties_build()
        pool = LoadbalancerPool(loadbalancer_pool_properties=props,
                                id_perms=id_perms,
                                display_name='pool1')
        pool.parent_uuid = tenant_id
        pool.uuid = pool_id
        return pool

    def _create_member(self, member_id, pool_id):
        id_perms = IdPermsType(uuid=member_id, enable=True)
        props = self._loadbalancer_member_properties_build()
        member = LoadbalancerMember(loadbalancer_member_properties=props,
                                    id_perms=id_perms)
        member.uuid = member_id
        member.parent_uuid = pool_id
        return member

    def _create_monitor(self, health_monitor_id, tenant_id):
        id_perms = IdPermsType(uuid=health_monitor_id, enable=True)
        props = self._loadbalancer_healthmonitor_properties_build()
        monitor = LoadbalancerHealthmonitor(
            health_monitor_id,
            loadbalancer_healthmonitor_properties=props,
            id_perms=id_perms)
        monitor.parent_uuid = tenant_id
        monitor.uuid = health_monitor_id
        return monitor

    def test_vip_list(self):
        def get_vips(*args, **kwargs):
            return self.loadbalancer.get_vips(*args, **kwargs)

        self.plugin.return_value.get_vips.side_effect = get_vips

        virtual_ip_list = [
            {'fq_name': ['default-domain', 'test', 'vip1'], 'uuid': _uuid()},
            {'fq_name': ['default-domain', 'test', 'vip2'], 'uuid': _uuid()}
            ]
        instance = self.api_server
        instance.virtual_ips_list.return_value = {
            'virtual-ips': virtual_ip_list
        }

        def virtual_ip_read(*args, **kwargs):
            props = self._virtual_ip_properties_build()
            vip = VirtualIp(virtual_ip_properties=props)
            vip.uuid = kwargs['id']
            vip.parent_uuid = _uuid()
            return vip

        instance.virtual_ip_read.side_effect = virtual_ip_read

        res = self.api.get(_get_path('lb/vips', fmt=self.fmt))
        self.assertEqual(res.status_int, exc.HTTPOk.code)

        res = self.deserialize(res)
        self.assertIn('vips', res)
        self.assertEqual(2, len(res['vips']))

    def test_vip_list_w_filter(self):
        def get_vips(*args, **kwargs):
            return self.loadbalancer.get_vips(*args, **kwargs)

        self.plugin.return_value.get_vips.side_effect = get_vips

        vip1_id = _uuid()
        vip2_id = _uuid()
        virtual_ip_list = [
            {'fq_name': ['default-domain', 'test', 'vip1'], 'uuid': vip1_id},
            {'fq_name': ['default-domain', 'test', 'vip2'], 'uuid': vip2_id}
            ]
        instance = self.api_server
        instance.virtual_ips_list.return_value = {
            'virtual-ips': virtual_ip_list
        }

        def virtual_ip_read(*args, **kwargs):
            name = None
            props = VirtualIpType()
            props.address = '127.0.0.1'
            if kwargs['id'] == vip1_id:
                name = 'vip1'
                props.protocol = 'HTTP'
            elif kwargs['id'] == vip2_id:
                name = 'vip2'
                props.protocol = 'HTTPS'
            props.protocol_port = 80
            props.connection_limit = 100
            props.subnet_id = _uuid()
            props.admin_state = True
            vip = VirtualIp(virtual_ip_properties=props, display_name=name)
            vip.uuid = kwargs['id']
            vip.parent_uuid = _uuid()
            return vip

        instance.virtual_ip_read.side_effect = virtual_ip_read

        res = self.api.get(_get_path('lb/vips', fmt=self.fmt),
                           {'protocol': ['HTTP']})
        self.assertEqual(res.status_int, exc.HTTPOk.code)

        res = self.deserialize(res)
        self.assertIn('vips', res)
        self.assertEqual(1, len(res['vips']))
        self.assertEqual('vip1', res['vips'][0]['name'])

    def test_vip_update(self):
        def update_vip(*args, **kwargs):
            return self.loadbalancer.update_vip(*args, **kwargs)

        self.plugin.return_value.update_vip.side_effect = update_vip

        vip_id = _uuid()

        id_perms = IdPermsType(uuid=vip_id, enable=True)
        props = self._virtual_ip_properties_build()
        vip = VirtualIp(virtual_ip_properties=props, id_perms=id_perms,
                        display_name='vip1')
        vip.parent_uuid = _uuid()
        vip.uuid = vip_id

        instance = self.api_server
        instance.virtual_ip_read.return_value = vip

        update_data = {'vip': {'admin_state_up': False}}

        res = self.api.put(_get_path('lb/vips', id=vip_id, fmt=self.fmt),
                           self.serialize(update_data))

        instance.virtual_ip_update.assert_called_with(mock.ANY)

        self.assertEqual(res.status_int, exc.HTTPOk.code)
        res = self.deserialize(res)
        self.assertIn('vip', res)

        return_value = {'name': 'vip1',
                        'admin_state_up': False,
                        'tenant_id': vip.parent_uuid,
                        'status': "ACTIVE",
                        'id': vip_id}
        self.assertDictContainsSubset(return_value, res['vip'])

    def test_vip_get(self):
        def get_vip(*args, **kwargs):
            return self.loadbalancer.get_vip(*args, **kwargs)

        self.plugin.return_value.get_vip.side_effect = get_vip

        vip_id = _uuid()

        id_perms = IdPermsType(uuid=vip_id, enable=True)
        props = self._virtual_ip_properties_build()
        vip = VirtualIp(virtual_ip_properties=props, id_perms=id_perms,
                        display_name='vip1')
        vip.parent_uuid = _uuid()
        vip.uuid = vip_id

        instance = self.api_server
        instance.virtual_ip_read.return_value = vip

        return_value = {'name': 'vip1',
                        'admin_state_up': True,
                        'tenant_id': vip.parent_uuid,
                        'status': "ACTIVE",
                        'id': vip_id}

        res = self.api.get(_get_path('lb/vips', id=vip_id, fmt=self.fmt))

        self.assertEqual(res.status_int, exc.HTTPOk.code)
        res = self.deserialize(res)
        self.assertIn('vip', res)
        self.assertDictContainsSubset(return_value, res['vip'])

    def test_vip_delete(self):
        def delete_vip(*args, **kwargs):
            return self.loadbalancer.delete_vip(*args, **kwargs)

        self.plugin.return_value.delete_vip.side_effect = delete_vip

        vip_id = _uuid()
        res = self.api.delete(_get_path('lb/vips', id=vip_id, fmt=self.fmt))
        self.assertEqual(res.status_int, exc.HTTPNoContent.code)

        instance = self.api_server
        instance.virtual_ip_delete.assert_called_with(id=vip_id)

    def test_pool_create(self):
        def create_pool(*args, **kwargs):
            return self.loadbalancer.create_pool(*args, **kwargs)

        def api_loadbalancer_pool_create(*args, **kwargs):
            pool = args[0]
            pool.parent_uuid = self._project.uuid
            return pool.uuid

        def healthmonitor_read(*args, **kwargs):
            hm = LoadbalancerHealthmonitor('hm1', self._project)
            hm.uuid = kwargs['id']
            return hm

        self.plugin.return_value.create_pool.side_effect = create_pool

        hm_id = _uuid()
        data = {'pool': {'name': 'pool1',
                         'description': 'descr_pool1',
                         'subnet_id': _uuid(),
                         'protocol': 'HTTP',
                         'lb_method': 'ROUND_ROBIN',
                         'health_monitors': [hm_id],
                         'admin_state_up': True,
                         'tenant_id': _uuid()}}

        instance = self.api_server
        instance.fq_name_to_id.return_value = None
        instance.project_read.side_effect = self._project_read

        instance.loadbalancer_pool_create.side_effect = \
            api_loadbalancer_pool_create

        instance.loadbalancer_healthmonitor_read.side_effect = \
            healthmonitor_read
        res = self.api.post(_get_path('lb/pools', fmt=self.fmt),
                            self.serialize(data),
                            content_type='application/%s' % self.fmt)

        self.assertEqual(res.status_int, exc.HTTPCreated.code)
        res = self.deserialize(res)
        self.assertIn('pool', res)

        return_value = copy.copy(data['pool'])
        return_value['provider'] = 'lbaas'
        return_value.update({'status': "ACTIVE", 'id': res['pool']['id']})
        self.assertEqual(return_value, res['pool'])

    def test_pool_list(self):
        def get_pools(*args, **kwargs):
            return self.loadbalancer.get_pools(*args, **kwargs)

        self.plugin.return_value.get_pools.side_effect = get_pools

        loadbalancer_pool_list = [
            {'fq_name': ['default-domain', 'test', 'pool1'], 'uuid': _uuid()},
            {'fq_name': ['default-domain', 'test', 'pool2'], 'uuid': _uuid()},
        ]
        instance = self.api_server
        instance.loadbalancer_pools_list.return_value = {
            'loadbalancer-pools': loadbalancer_pool_list
        }

        def loadbalancer_pool_read(*args, **kwargs):
            props = self._loadbalancer_pool_properties_build()
            pool = LoadbalancerPool(loadbalancer_pool_properties=props)
            pool.uuid = kwargs['id']
            pool.parent_uuid = _uuid()
            return pool

        instance.loadbalancer_pool_read.side_effect = loadbalancer_pool_read

        res = self.api.get(_get_path('lb/pools', fmt=self.fmt))
        self.assertEqual(res.status_int, exc.HTTPOk.code)

        res = self.deserialize(res)
        self.assertIn('pools', res)
        self.assertEqual(2, len(res['pools']))

    def test_pool_update(self):
        def update_pool(*args, **kwargs):
            return self.loadbalancer.update_pool(*args, **kwargs)

        self.plugin.return_value.update_pool.side_effect = update_pool

        pool_id = _uuid()
        pool = self._create_pool(pool_id)

        instance = self.api_server
        instance.loadbalancer_pool_read.return_value = pool

        update_data = {'pool': {'admin_state_up': False}}

        res = self.api.put(_get_path('lb/pools', id=pool_id, fmt=self.fmt),
                           self.serialize(update_data))

        instance.loadbalancer_pool_update.assert_called_with(mock.ANY)

        self.assertEqual(res.status_int, exc.HTTPOk.code)
        res = self.deserialize(res)
        self.assertIn('pool', res)

        return_value = {'name': 'pool1',
                        'admin_state_up': False,
                        'tenant_id': pool.parent_uuid,
                        'status': "ACTIVE",
                        'id': pool_id}
        self.assertDictContainsSubset(return_value, res['pool'])

    def test_pool_get(self):
        def get_pool(*args, **kwargs):
            return self.loadbalancer.get_pool(*args, **kwargs)

        self.plugin.return_value.get_pool.side_effect = get_pool

        pool_id = _uuid()
        pool = self._create_pool(pool_id)

        instance = self.api_server
        instance.loadbalancer_pool_read.return_value = pool

        res = self.api.get(_get_path('lb/pools', id=pool_id, fmt=self.fmt))

        self.assertEqual(res.status_int, exc.HTTPOk.code)
        res = self.deserialize(res)
        self.assertIn('pool', res)
        return_value = {'name': 'pool1',
                        'admin_state_up': True,
                        'tenant_id': pool.parent_uuid,
                        'status': "ACTIVE",
                        'id': pool_id}
        self.assertDictContainsSubset(return_value, res['pool'])

    def test_pool_delete(self):
        def delete_pool(*args, **kwargs):
            return self.loadbalancer.delete_pool(*args, **kwargs)

        self.plugin.return_value.delete_pool.side_effect = delete_pool

        pool_id = _uuid()
        res = self.api.delete(_get_path('lb/pools', id=pool_id, fmt=self.fmt))
        self.assertEqual(res.status_int, exc.HTTPNoContent.code)

        instance = self.api_server
        instance.loadbalancer_pool_delete.assert_called_with(id=pool_id)

    def test_member_create(self):
        def create_member(*args, **kwargs):
            return self.loadbalancer.create_member(*args, **kwargs)

        self.plugin.return_value.create_member.side_effect = create_member

        pool_id = _uuid()
        pool = self._create_pool(pool_id)

        def api_loadbalancer_member_create(*args, **kwargs):
            mbr = args[0]
            mbr.parent_uuid = pool_id
            return mbr.uuid

        instance = self.api_server
        instance.fq_name_to_id.return_value = None
        instance.loadbalancer_pool_read.return_value = pool
        instance.loadbalancer_member_create.side_effect = \
            api_loadbalancer_member_create

        data = {'member': {'pool_id': pool_id,
                           'address': '127.0.0.1',
                           'protocol_port': 80,
                           'weight': 1,
                           'admin_state_up': True,
                           'tenant_id': pool.parent_uuid}}

        res = self.api.post(_get_path('lb/members', fmt=self.fmt),
                            self.serialize(data),
                            content_type='application/%s' % self.fmt)

        self.assertEqual(res.status_int, exc.HTTPCreated.code)
        res = self.deserialize(res)
        self.assertIn('member', res)

        return_value = copy.copy(data['member'])
        return_value.update({'status': "ACTIVE", 'id': res['member']['id']})

        self.assertEqual(return_value, res['member'])

    def test_member_list(self):
        def get_members(*args, **kwargs):
            return self.loadbalancer.get_members(*args, **kwargs)

        self.plugin.return_value.get_members.side_effect = get_members

        uuid1 = _uuid()
        uuid2 = _uuid()
        loadbalancer_member_list = [
            {'fq_name': ['default-domain', 'test', uuid1], 'uuid': uuid1},
            {'fq_name': ['default-domain', 'test', uuid2], 'uuid': uuid2},
        ]

        instance = self.api_server
        instance.loadbalancer_members_list.return_value = {
            'loadbalancer-members': loadbalancer_member_list
        }

        pool_id = _uuid()
        pool = self._create_pool(pool_id)

        instance.loadbalancer_pool_read.return_value = pool

        def loadbalancer_member_read(*args, **kwargs):
            member = self._create_member(kwargs['id'], pool_id)
            return member

        instance.loadbalancer_member_read.side_effect = \
            loadbalancer_member_read

        res = self.api.get(_get_path('lb/members', fmt=self.fmt))

        self.assertEqual(res.status_int, exc.HTTPOk.code)

        res = self.deserialize(res)
        self.assertIn('members', res)
        self.assertEqual(2, len(res['members']))

    def test_member_update(self):
        def update_member(*args, **kwargs):
            return self.loadbalancer.update_member(*args, **kwargs)

        self.plugin.return_value.update_member.side_effect = update_member

        member_id = _uuid()
        pool_id = _uuid()

        pool = self._create_pool(pool_id)
        member = self._create_member(member_id, pool_id)

        instance = self.api_server
        instance.loadbalancer_pool_read.return_value = pool
        instance.loadbalancer_member_read.return_value = member

        update_data = {'member': {'admin_state_up': False}}

        res = self.api.put(_get_path('lb/members', id=member_id,
                                     fmt=self.fmt),
                           self.serialize(update_data))

        instance.loadbalancer_member_update.assert_called_with(mock.ANY)

        self.assertEqual(res.status_int, exc.HTTPOk.code)
        res = self.deserialize(res)
        self.assertIn('member', res)
        return_value = {'admin_state_up': False,
                        'tenant_id': pool.parent_uuid,
                        'status': "ACTIVE",
                        'id': member_id}

        self.assertDictContainsSubset(return_value, res['member'])

    def test_member_get(self):
        def get_member(*args, **kwargs):
            return self.loadbalancer.get_member(*args, **kwargs)

        self.plugin.return_value.get_member.side_effect = get_member

        member_id = _uuid()
        pool_id = _uuid()

        pool = self._create_pool(pool_id)
        member = self._create_member(member_id, pool_id)

        instance = self.api_server
        instance.loadbalancer_pool_read.return_value = pool
        instance.loadbalancer_member_read.return_value = member

        res = self.api.get(_get_path('lb/members', id=member_id,
                                     fmt=self.fmt))

        self.assertEqual(res.status_int, exc.HTTPOk.code)
        res = self.deserialize(res)
        self.assertIn('member', res)
        return_value = {'admin_state_up': True,
                        'tenant_id': pool.parent_uuid,
                        'status': "ACTIVE",
                        'id': member_id}
        self.assertDictContainsSubset(return_value, res['member'])

    def test_member_delete(self):
        def delete_member(*args, **kwargs):
            return self.loadbalancer.delete_member(*args, **kwargs)

        self.plugin.return_value.delete_member.side_effect = delete_member

        member_id = _uuid()
        res = self.api.delete(_get_path('lb/members', id=member_id,
                                        fmt=self.fmt))
        self.assertEqual(res.status_int, exc.HTTPNoContent.code)

        instance = self.api_server
        instance.loadbalancer_member_delete.assert_called_with(id=member_id)

    def test_health_monitor_create(self):
        def create_health_monitor(*args, **kwargs):
            return self.loadbalancer.create_health_monitor(*args, **kwargs)

        def api_loadbalancer_healthmonitor_create(*args, **kwargs):
            monitor = args[0]
            monitor.parent_uuid = self._project.uuid
            return monitor.uuid

        self.plugin.return_value.create_health_monitor.side_effect = \
            create_health_monitor

        instance = self.api_server
        instance.project_read.side_effect = self._project_read

        instance.loadbalancer_healthmonitor_create.side_effect = \
            api_loadbalancer_healthmonitor_create

        data = {'health_monitor': {'type': 'HTTP',
                                   'delay': 2,
                                   'timeout': 1,
                                   'max_retries': 3,
                                   'http_method': 'GET',
                                   'url_path': '/path',
                                   'expected_codes': '200-300',
                                   'admin_state_up': True,
                                   'tenant_id': _uuid()}}

        res = self.api.post(_get_path('lb/health_monitors',
                                      fmt=self.fmt),
                            self.serialize(data),
                            content_type='application/%s' % self.fmt)

        self.assertEqual(res.status_int, exc.HTTPCreated.code)
        res = self.deserialize(res)
        self.assertIn('health_monitor', res)

        health_monitor_id = res['health_monitor']['id']
        return_value = copy.copy(data['health_monitor'])
        return_value.update({'status': "ACTIVE", 'id': health_monitor_id})
        self.assertEqual(return_value, res['health_monitor'])

    def test_health_monitor_list(self):
        def get_health_monitors(*args, **kwargs):
            return self.loadbalancer.get_health_monitors(*args, **kwargs)

        self.plugin.return_value.get_health_monitors.side_effect = \
            get_health_monitors

        uuid1 = _uuid()
        uuid2 = _uuid()
        health_monitor_list = [
            {'fq_name': ['default-domain', 'test', uuid1], 'uuid': uuid1},
            {'fq_name': ['default-domain', 'test', uuid2], 'uuid': uuid2},
        ]
        instance = self.api_server
        instance.loadbalancer_healthmonitors_list.return_value = {
            'loadbalancer-healthmonitors': health_monitor_list
        }

        def loadbalancer_healthmonitor_read(*args, **kwargs):
            monitor = self._create_monitor(kwargs['id'], _uuid())
            return monitor

        instance.loadbalancer_healthmonitor_read.side_effect = \
            loadbalancer_healthmonitor_read

        res = self.api.get(_get_path('lb/health_monitors', fmt=self.fmt))

        res = self.deserialize(res)
        self.assertIn('health_monitors', res)
        self.assertEqual(2, len(res['health_monitors']))

    def test_health_monitor_update(self):
        def update_health_monitor(*args, **kwargs):
            return self.loadbalancer.update_health_monitor(*args, **kwargs)

        self.plugin.return_value.update_health_monitor.side_effect = \
            update_health_monitor

        health_monitor_id = _uuid()
        monitor = self._create_monitor(health_monitor_id, _uuid())

        instance = self.api_server
        instance.loadbalancer_healthmonitor_read.return_value = monitor

        update_data = {'health_monitor': {'admin_state_up': False}}

        res = self.api.put(_get_path('lb/health_monitors',
                                     id=health_monitor_id,
                                     fmt=self.fmt),
                           self.serialize(update_data))

        instance.loadbalancer_healthmonitor_update.assert_called_with(mock.ANY)
        self.assertEqual(res.status_int, exc.HTTPOk.code)
        res = self.deserialize(res)
        self.assertIn('health_monitor', res)

        return_value = {'type': 'HTTP',
                        'admin_state_up': False,
                        'tenant_id': monitor.parent_uuid,
                        'status': "ACTIVE",
                        'id': health_monitor_id}

        self.assertDictContainsSubset(return_value, res['health_monitor'])

    def test_health_monitor_get(self):
        def get_health_monitor(*args, **kwargs):
            return self.loadbalancer.get_health_monitor(*args, **kwargs)

        self.plugin.return_value.get_health_monitor.side_effect = \
            get_health_monitor

        health_monitor_id = _uuid()
        monitor = self._create_monitor(health_monitor_id, _uuid())

        instance = self.api_server
        instance.loadbalancer_healthmonitor_read.return_value = monitor

        res = self.api.get(_get_path('lb/health_monitors',
                                     id=health_monitor_id,
                                     fmt=self.fmt))

        self.assertEqual(res.status_int, exc.HTTPOk.code)
        res = self.deserialize(res)
        self.assertIn('health_monitor', res)

        return_value = {'type': 'HTTP',
                        'admin_state_up': True,
                        'tenant_id': monitor.parent_uuid,
                        'status': "ACTIVE",
                        'id': health_monitor_id}

        self.assertDictContainsSubset(return_value, res['health_monitor'])

    def test_health_monitor_delete(self):
        def delete_health_monitor(*args, **kwargs):
            return self.loadbalancer.delete_health_monitor(*args, **kwargs)

        self.plugin.return_value.delete_health_monitor.side_effect = \
            delete_health_monitor

        health_monitor_id = _uuid()
        res = self.api.delete(_get_path('lb/health_monitors',
                                        id=health_monitor_id,
                                        fmt=self.fmt))
        self.assertEqual(res.status_int, exc.HTTPNoContent.code)

        instance = self.api_server
        instance.loadbalancer_healthmonitor_delete.assert_called_with(
            id=health_monitor_id)

    def test_create_pool_health_monitor(self):
        def create_pool_health_monitor(*args, **kwargs):
            return self.loadbalancer.create_pool_health_monitor(
                *args, **kwargs)

        self.plugin.return_value.create_pool_health_monitor.side_effect = \
            create_pool_health_monitor

        instance = self.api_server
        instance.project_read.side_effect = self._project_read

        tenant_id = _uuid()
        pool_id = _uuid()
        health_monitor_id = _uuid()

        pool = self._create_pool(pool_id, tenant_id)
        monitor = self._create_monitor(health_monitor_id, tenant_id)
        instance.loadbalancer_pool_read.return_value = pool
        instance.loadbalancer_healthmonitor_read.return_value = monitor

        data = {'health_monitor': {'id': health_monitor_id,
                                   'tenant_id': tenant_id}}

        res = self.api.post('/lb/pools/%s/health_monitors' % pool_id,
                            self.serialize(data),
                            content_type='application/%s' % self.fmt)

        self.assertEqual(res.status_int, exc.HTTPCreated.code)
        res = self.deserialize(res)
        self.assertIn('health_monitor', res)
        return_value = copy.copy(data['health_monitor'])
        self.assertEqual(return_value, res['health_monitor'])

    def test_delete_pool_health_monitor(self):
        def delete_pool_health_monitor(*args, **kwargs):
            return self.loadbalancer.delete_pool_health_monitor(
                *args, **kwargs)

        self.plugin.return_value.delete_pool_health_monitor.side_effect = \
            delete_pool_health_monitor

        tenant_id = _uuid()
        pool_id = _uuid()
        health_monitor_id = _uuid()

        pool = self._create_pool(pool_id, tenant_id)
        monitor = self._create_monitor(health_monitor_id, tenant_id)
        monitor.fq_name = ['default-domain', 'test', health_monitor_id]
        pool.add_loadbalancer_healthmonitor(monitor)

        instance = self.api_server
        instance.loadbalancer_pool_read.return_value = pool
        instance.loadbalancer_healthmonitor_read.return_value = monitor

        res = self.api.delete('/lb/pools/%s/health_monitors/%s' %
                              (pool_id, health_monitor_id))

        instance.loadbalancer_pool_update.assert_called_with(mock.ANY)
        self.assertEqual(res.status_int, exc.HTTPNoContent.code)



from abc import abstractmethod

from neutron.api.v2 import attributes as attr
from neutron.api.v2 import base
from neutron.common import exceptions as qexception
from neutron.api import extensions
from neutron import manager
from oslo.config import cfg


# Ipam Exceptions
class IpamNotFound(qexception.NotFound):
    message = _("IPAM %(id)s could not be found")

# Attribute Map
RESOURCE_ATTRIBUTE_MAP = {
    'ipams': {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:regex': attr.UUID_PATTERN},
               'is_visible': True},
        'name': {'allow_post': True, 'allow_put': False,
                 'is_visible': True, 'default': ''},
        'fq_name': {'allow_post': False, 'allow_put': False,
                    'is_visible': True},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'required_by_policy': True,
                      'is_visible': True},
        'mgmt': {'allow_post': True, 'allow_put': True,
                 'is_visible': True, 'default': None},
        'nets_using': {'allow_post': False, 'allow_put': False,
                       'is_visible': True, 'default': ''}
    },
}


class Ipam(object):

    @classmethod
    def get_name(cls):
        return "Network IP Address Management"

    @classmethod
    def get_alias(cls):
        return "ipam"

    @classmethod
    def get_description(cls):
        return ("Configuration object for holding common to a set of"
                " IP address blocks")

    @classmethod
    def get_namespace(cls):
        return "http://docs.openstack.org/TODO"

    @classmethod
    def get_updated(cls):
        return "2012-07-20T10:00:00-00:00"

    @classmethod
    def get_resources(cls):
        """ Returns Ext Resources """
        exts = []
        plugin = manager.NeutronManager.get_plugin()
        for resource_name in ['ipam']:
            collection_name = resource_name + "s"
            params = RESOURCE_ATTRIBUTE_MAP.get(collection_name, dict())

            member_actions = {}

            controller = base.create_resource(collection_name,
                                              resource_name,
                                              plugin, params,
                                              member_actions=member_actions)

            ex = extensions.ResourceExtension(collection_name,
                                              controller,
                                              member_actions=member_actions)
            exts.append(ex)

        return exts

# end class Ipam


class IpamPluginBase(object):

    @abstractmethod
    def create_ipam(self, context, ipam):
        pass

    @abstractmethod
    def update_ipam(self, context, id, ipam):
        pass

    @abstractmethod
    def get_ipam(self, context, id, fields=None):
        pass

    @abstractmethod
    def delete_ipam(self, context, id):
        pass

    @abstractmethod
    def get_ipams(self, context, filters=None, fields=None):
        pass
# end class IpamPluginBase



#
# Copyright (c) 2014 Juniper Networks, Inc. All rights reserved.
#

import uuid

from neutron.extensions import loadbalancer
from neutron.openstack.common import uuidutils
from vnc_api.vnc_api import IdPermsType, NoIdError
from vnc_api.vnc_api import LoadbalancerHealthmonitor
from vnc_api.vnc_api import LoadbalancerHealthmonitorType

from resource_manager import ResourceManager


class LoadbalancerHealthmonitorManager(ResourceManager):
    _loadbalancer_health_type_mapping = {
        'admin_state': 'admin_state_up',
        'monitor_type': 'type',
        'delay': 'delay',
        'timeout': 'timeout',
        'max_retries': 'max_retries',
        'http_method': 'http_method',
        'url_path': 'url_path',
        'expected_codes': 'expected_codes'
    }

    @property
    def property_type_mapping(self):
        return self._loadbalancer_health_type_mapping

    def make_properties(self, health_monitor):
        props = LoadbalancerHealthmonitorType()
        for key, mapping in self._loadbalancer_health_type_mapping.iteritems():
            if mapping in health_monitor:
                setattr(props, key, health_monitor[mapping])
        return props

    def make_dict(self, health_monitor, fields=None):
        res = {'id': health_monitor.uuid,
               'tenant_id': health_monitor.parent_uuid.replace('-', ''),
               'status': self._get_object_status(health_monitor)}

        props = health_monitor.get_loadbalancer_healthmonitor_properties()
        for key, mapping in self._loadbalancer_health_type_mapping.iteritems():
            value = getattr(props, key)
            if value is not None:
                res[mapping] = value

        pool_ids = []
        pool_back_refs = health_monitor.get_loadbalancer_pool_back_refs()
        for pool_back_ref in pool_back_refs or []:
            pool_id = {}
            pool_id['pool_id'] = pool_back_ref['uuid']
            pool_ids.append(pool_id)
        res['pools'] = pool_ids

        return self._fields(res, fields)

    def resource_read(self, id):
        return self._api.loadbalancer_healthmonitor_read(id=id)

    def resource_list(self, tenant_id=None):
        if tenant_id:
            parent_id = str(uuid.UUID(tenant_id))
        else:
            parent_id = None
        return self._api.loadbalancer_healthmonitors_list(parent_id=parent_id)

    def resource_update(self, obj):
        return self._api.loadbalancer_healthmonitor_update(obj)

    def resource_delete(self, id):
        return self._api.loadbalancer_healthmonitor_delete(id=id)

    def get_exception_notfound(self, id=None):
        return loadbalancer.HealthMonitorNotFound(monitor_id=id)

    def get_exception_inuse(self, id=None):
        return loadbalancer.HealthMonitorInUse(monitor_id=id)

    @property
    def neutron_name(self):
        return "health_monitor"

    @property
    def resource_name_plural(self):
        return "loadbalancer-healthmonitors"

    def create(self, context, health_monitor):
        """
        Create a loadbalancer_healtmonitor object.
        """
        m = health_monitor['health_monitor']
        tenant_id = self._get_tenant_id_for_create(context, m)
        project = self._project_read(project_id=tenant_id)

        uuid = uuidutils.generate_uuid()
        props = self.make_properties(m)
        id_perms = IdPermsType(enable=True)
        monitor_db = LoadbalancerHealthmonitor(
            uuid, project, loadbalancer_healthmonitor_properties=props,
            id_perms=id_perms)
        monitor_db.uuid = uuid

        self._api.loadbalancer_healthmonitor_create(monitor_db)
        return self.make_dict(monitor_db)

    def update_properties(self, monitor_db, id, m):
        props = monitor_db.get_loadbalancer_healthmonitor_properties()
        if self.update_properties_subr(props, m):
            monitor_db.set_loadbalancer_healthmonitor_properties(props)
            return True
        return False



# Copyright 2014 Juniper Networks.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import datetime
import json
import uuid

import mock
import netaddr
from oslo.config import cfg
from testtools import matchers
import webob.exc

from neutron.api import extensions
from neutron.api.v2 import attributes as attr
from neutron.api.v2 import base as api_base
from neutron.common import exceptions as exc
from neutron import context as neutron_context
from neutron.db import api as db
from neutron.db import db_base_plugin_v2
from neutron.db import external_net_db
from neutron.db import l3_db
from neutron.db import quota_db  # noqa
from neutron.db import securitygroups_db
from neutron.extensions import portbindings
from neutron.extensions import securitygroup as ext_sg
from neutron.tests.unit import _test_extension_portbindings as test_bindings
from neutron.tests.unit import test_db_plugin as test_plugin
from neutron.tests.unit import test_extension_security_group as test_sg
from neutron.tests.unit import test_extensions
from neutron.tests.unit import test_l3_plugin


CONTRAIL_PKG_PATH = "neutron_plugin_contrail.plugins.opencontrail.contrail_plugin_core"


class FakeServer(db_base_plugin_v2.NeutronDbPluginV2,
                 external_net_db.External_net_db_mixin,
                 securitygroups_db.SecurityGroupDbMixin,
                 l3_db.L3_NAT_db_mixin):
    """FakeServer for contrail api server.

    This class mocks behaviour of contrail API server.
    """
    supported_extension_aliases = ['external-net', 'router', 'floatingip']

    @property
    def _core_plugin(self):
        return self

    def create_port(self, context, port):
        self._ensure_default_security_group_on_port(context, port)
        sgids = self._get_security_groups_on_port(context, port)
        result = super(FakeServer, self).create_port(context, port)
        self._process_port_create_security_group(context, result, sgids)
        return result

    def update_port(self, context, id, port):
        original_port = self.get_port(context, id)
        updated_port = super(FakeServer, self).update_port(context, id, port)
        port_updates = port['port']
        if ext_sg.SECURITYGROUPS in port_updates:
            port_updates[ext_sg.SECURITYGROUPS] = (
                self._get_security_groups_on_port(context, port))
            self._delete_port_security_group_bindings(context, id)
            self._process_port_create_security_group(
                context,
                updated_port,
                port_updates[ext_sg.SECURITYGROUPS])
        else:
            updated_port[ext_sg.SECURITYGROUPS] = (
                original_port[ext_sg.SECURITYGROUPS])

        return updated_port

    def delete_port(self, context, id, l3_port_check=True):
        if l3_port_check:
            self.prevent_l3_port_deletion(context, id)
        self.disassociate_floatingips(context, id)
        super(FakeServer, self).delete_port(context, id)

    def create_network(self, context, network):
        net_data = network['network']
        tenant_id = self._get_tenant_id_for_create(context, net_data)
        self._ensure_default_security_group(context, tenant_id)
        result = super(FakeServer, self).create_network(context, network)
        self._process_l3_create(context, result, network['network'])
        return result

    def update_network(self, context, id, network):
        with context.session.begin(subtransactions=True):
            result = super(
                FakeServer, self).update_network(context, id, network)
            self._process_l3_update(context, result, network['network'])
        return result

    def delete_network(self, context, id):
        self.delete_disassociated_floatingips(context, id)
        super(FakeServer, self).delete_network(context, id)

    def request(self, *args, **kwargs):
        request_data = json.loads(kwargs['data'])
        context_dict = request_data['context']
        context = neutron_context.Context.from_dict(context_dict)
        resource_type = context_dict['type']
        operation = context_dict['operation']
        data = request_data['data']
        resource = None
        if data.get('resource'):
            body = data['resource']
            if resource_type not in [
                    'security_group_rule', 'router', 'floatingip']:
                for key, value in body.items():
                    if value is None:
                        body[key] = attr.ATTR_NOT_SPECIFIED
            resource = {resource_type: body}

        obj = {}
        code = webob.exc.HTTPOk.code
        try:
            if operation == 'READ':
                func = getattr(self, 'get_%s' % resource_type)
                obj = func(context, data['id'])
            if operation == 'READALL':
                func = getattr(self, 'get_%ss' % resource_type)
                obj = func(context, filters=data.get('filters'))
            if operation == 'READCOUNT':
                func = getattr(self, 'get_%ss_count' % resource_type)
                count = func(context, filters=data.get('filters'))
                obj = {'count': count}
            if operation == 'CREATE':
                func = getattr(self, 'create_%s' % resource_type)
                obj = func(context, resource)
            if operation == 'UPDATE':
                func = getattr(self, 'update_%s' % resource_type)
                obj = func(context, data['id'], resource)
            if operation == 'DELETE':
                func = getattr(self, 'delete_%s' % resource_type)
                obj = func(context, data['id'])
            if operation == 'ADDINTERFACE':
                obj = self.add_router_interface(
                    context, data['id'], data['resource'])
            if operation == 'DELINTERFACE':
                obj = self.remove_router_interface(
                    context, data['id'], data['resource'])
        except (exc.NeutronException,
                netaddr.AddrFormatError) as e:
            for fault in api_base.FAULT_MAP:
                if isinstance(e, fault):
                    mapped_exc = api_base.FAULT_MAP[fault]
                    code = mapped_exc.code
            obj = {'type': e.__class__.__name__,
                   'message': e.msg, 'detail': ''}
            if data.get('id'):
                obj['id'] = data.get('id')
        response = mock.MagicMock()
        response.status_code = code

        def return_obj():
            return obj
        response.json = return_obj
        return response


FAKE_SERVER = FakeServer()


class Context(object):
    def __init__(self, tenant_id=''):
        self.read_only = False
        self.show_deleted = False
        self.roles = [u'admin', u'KeystoneServiceAdmin', u'KeystoneAdmin']
        self._read_deleted = 'no'
        self.timestamp = datetime.datetime.now()
        self.auth_token = None
        self._session = None
        self._is_admin = True
        self.admin = uuid.uuid4().hex.decode()
        self.request_id = 'req-' + str(uuid.uuid4())
        self.tenant = tenant_id


class KeyStoneInfo(object):
    """To generate Keystone Authentication information
       Contrail Driver expects Keystone auth info for testing purpose.
    """
    auth_protocol = 'http'
    auth_host = 'host'
    auth_port = 5000
    admin_user = 'neutron'
    admin_password = 'neutron'
    admin_token = 'neutron'
    admin_tenant_name = 'neutron'


class JVContrailPluginTestCase(test_plugin.NeutronDbPluginV2TestCase):
    _plugin_name = ('%s.NeutronPluginContrailCoreV2' % CONTRAIL_PKG_PATH)

    def setUp(self, plugin=None, ext_mgr=None):

        cfg.CONF.keystone_authtoken = KeyStoneInfo()
        mock.patch('requests.post').start().side_effect = FAKE_SERVER.request
        db.configure_db()
        super(JVContrailPluginTestCase, self).setUp(self._plugin_name)


class TestContrailNetworksV2(test_plugin.TestNetworksV2,
                             JVContrailPluginTestCase):
    def setUp(self):
        super(TestContrailNetworksV2, self).setUp()


class TestContrailSubnetsV2(test_plugin.TestSubnetsV2,
                            JVContrailPluginTestCase):
    def setUp(self):
        super(TestContrailSubnetsV2, self).setUp()

    # Support ipv6 in contrail is planned in Juno
    def test_update_subnet_ipv6_attributes(self):
        self.skipTest("Contrail isn't supporting ipv6 yet")

    def test_update_subnet_ipv6_inconsistent_address_attribute(self):
        self.skipTest("Contrail isn't supporting ipv6 yet")

    def test_update_subnet_ipv6_inconsistent_enable_dhcp(self):
        self.skipTest("Contrail isn't supporting ipv6 yet")

    def test_update_subnet_ipv6_inconsistent_ra_attribute(self):
        self.skipTest("Contrail isn't supporting ipv6 yet")

    def test_delete_subnet_dhcp_port_associated_with_other_subnets(self):
        self.skipTest("There is no dhcp port in contrail")

    def _helper_test_validate_subnet(self, option, exception):
        cfg.CONF.set_override(option, 0)
        with self.network() as network:
            subnet = {'network_id': network['network']['id'],
                      'cidr': '10.0.2.0/24',
                      'ip_version': 4,
                      'tenant_id': network['network']['tenant_id'],
                      'gateway_ip': '10.0.2.1',
                      'dns_nameservers': ['8.8.8.8'],
                      'host_routes': [{'destination': '135.207.0.0/16',
                                       'nexthop': '1.2.3.4'}]}
            e = self.assertRaises(exception,
                                  FAKE_SERVER._validate_subnet,
                                  neutron_context.get_admin_context(
                                      load_admin_roles=False),
                                  subnet)
            self.assertThat(
                str(e),
                matchers.Not(matchers.Contains('built-in function id')))


class TestContrailPortsV2(test_plugin.TestPortsV2,
                          JVContrailPluginTestCase):
    def setUp(self):
        super(TestContrailPortsV2, self).setUp()

    def test_delete_ports_by_device_id(self):
        self.skipTest("This method tests rpc API of "
                      "which contrail isn't using")

    def test_delete_ports_by_device_id_second_call_failure(self):
        self.skipTest("This method tests rpc API of "
                      "which contrail isn't using")

    def test_delete_ports_ignores_port_not_found(self):
        self.skipTest("This method tests private method of "
                      "which contrail isn't using")


class TestContrailSecurityGroups(test_sg.TestSecurityGroups,
                                 JVContrailPluginTestCase):
    def setUp(self, plugin=None, ext_mgr=None):
        super(TestContrailSecurityGroups, self).setUp(self._plugin_name,
                                                      ext_mgr)
        ext_mgr = extensions.PluginAwareExtensionManager.get_instance()
        self.ext_api = test_extensions.setup_extensions_middleware(ext_mgr)


class TestContrailPortBinding(JVContrailPluginTestCase,
                              test_bindings.PortBindingsTestCase):
    from neutron_plugin_contrail.plugins.opencontrail.contrail_plugin_core import NeutronPluginContrailCoreV2
    VIF_TYPE = portbindings.VIF_TYPE_VROUTER
    HAS_PORT_FILTER = True

    def setUp(self):
        super(TestContrailPortBinding, self).setUp()


class TestContrailL3NatTestCase(JVContrailPluginTestCase,
                                test_l3_plugin.L3NatDBIntTestCase):
    mock_rescheduling = False

    def setUp(self):
        super(TestContrailL3NatTestCase, self).setUp()



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2012 OpenStack Foundation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from abc import ABCMeta
from abc import abstractmethod

from oslo.config import cfg

from neutron.api import extensions
from neutron.api.v2 import attributes as attr
from neutron.api.v2 import base
from neutron.common import exceptions as qexception
from neutron import manager
from neutron.openstack.common import uuidutils
from neutron import quota


# Route table Exceptions
class RouteTableNotFound(qexception.NotFound):
    message = _("Route table %(id)s does not exist")

# Attribute Map
RESOURCE_ATTRIBUTE_MAP = {
    'route_tables': {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:uuid': None},
               'is_visible': True,
               'primary_key': True},
        'name': {'allow_post': True, 'allow_put': False,
                 'is_visible': True, 'default': '',
                 'validate': {'type:name_not_default': None}},
        'fq_name': {'allow_post': True, 'allow_put': False,
                    'is_visible': True, 'default': '',
                    'validate': {'type:name_not_default': None}},
        'routes': {'allow_post': True, 'allow_put': True,
                   'is_visible': True, 'default': ''},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'required_by_policy': True,
                      'is_visible': True},
    },
    'nat_instances': {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:uuid': None},
               'is_visible': True,
               'primary_key': True},
        'name': {'allow_post': True, 'allow_put': False,
                 'is_visible': True, 'default': '',
                 'validate': {'type:name_not_default': None}},
        'fq_name': {'allow_post': True, 'allow_put': False,
                    'is_visible': True, 'default': '',
                    'validate': {'type:name_not_default': None}},
        'internal_net': {'allow_post': True, 'allow_put': False,
                         'is_visible': True, 'default': ''},
        'internal_ip': {'allow_post': True, 'allow_put': False,
                        'is_visible': True, 'default': ''},
        'external_net': {'allow_post': True, 'allow_put': False,
                         'is_visible': True, 'default': ''},
        'external_ip': {'allow_post': True, 'allow_put': False,
                        'is_visible': True, 'default': ''},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'required_by_policy': True,
                      'is_visible': True},
    },
}

EXTENDED_ATTRIBUTES_2_0 = {
    'networks': {
        'vpc:route_table': {'allow_post': True,
                            'allow_put': True,
                            'default': '',
                            'is_visible': True},
    }
}


class Vpcroutetable(extensions.ExtensionDescriptor):
    """ Route table extension"""

    @classmethod
    def get_name(cls):
        return "route-table"

    @classmethod
    def get_alias(cls):
        return "route-table"

    @classmethod
    def get_description(cls):
        return "VPC route tables extension."

    @classmethod
    def get_namespace(cls):
        # todo
        return "http://docs.openstack.org/ext/routetables/api/v2.0"

    @classmethod
    def get_updated(cls):
        return "2013-07-24T10:00:00-00:00"

    @classmethod
    def get_resources(cls):
        """ Returns Ext Resources """
        my_plurals = [(key, key[:-1]) for key in RESOURCE_ATTRIBUTE_MAP.keys()]
        attr.PLURALS.update(dict(my_plurals))
        exts = []
        plugin = manager.NeutronManager.get_plugin()
        for resource_name in ['route_table', 'nat_instance']:
            collection_name = resource_name.replace('_', '-') + "s"
            params = RESOURCE_ATTRIBUTE_MAP.get(resource_name + "s", dict())
            quota.QUOTAS.register_resource_by_name(resource_name)
            controller = base.create_resource(collection_name,
                                              resource_name,
                                              plugin, params, allow_bulk=True,
                                              allow_pagination=True,
                                              allow_sorting=True)

            ex = extensions.ResourceExtension(collection_name,
                                              controller,
                                              attr_map=params)
            exts.append(ex)

        return exts

    def get_extended_resources(self, version):
        if version == "2.0":
            return EXTENDED_ATTRIBUTES_2_0
        else:
            return {}


class RouteTablePluginBase(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def create_route_table(self, context, route_table):
        pass

    @abstractmethod
    def delete_route_table(self, context, id):
        pass

    @abstractmethod
    def update_route_table(self, context, id, route_table):
        pass

    @abstractmethod
    def get_route_tables(self, context, filters=None, fields=None,
                         sorts=None, limit=None, marker=None,
                         page_reverse=False):
        pass

    @abstractmethod
    def get_route_table(self, context, id, fields=None):
        pass

    @abstractmethod
    def create_nat_instance(self, context, nat_instance):
        pass

    @abstractmethod
    def delete_nat_instance(self, context, id):
        pass

    @abstractmethod
    def get_nat_instances(self, context, filters=None, fields=None,
                          sorts=None, limit=None, marker=None,
                          page_reverse=False):
        pass

    @abstractmethod
    def get_nat_instance(self, context, id, fields=None):
        pass



#
# Copyright (c) 2013 Juniper Networks, Inc. All rights reserved.
#

from setuptools import setup, find_packages

def requirements(filename):
    with open(filename) as f:
        lines = f.read().splitlines()
    return lines

setup(
    name='neutron_plugin_contrail',
    version='0.1.1',
    packages=find_packages(),
    package_data={'': ['*.html', '*.css', '*.xml']},
    zip_safe=False,
    long_description="Contrail neutron plugin",
    test_suite='neutron_plugin_contrail.tests',
    tests_require=requirements('test-requirements.txt'),
)



from abc import abstractmethod

from neutron.api.v2 import attributes as attr
from neutron.api.v2 import base
from neutron.common import exceptions as qexception
from neutron.api import extensions
from neutron import manager
from oslo.config import cfg


# Policy Exceptions
class PolicyNotFound(qexception.NotFound):
    message = _("Policy %(id)s could not be found")

# Attribute Map
RESOURCE_ATTRIBUTE_MAP = {
    'policys': {
        'id': {'allow_post': False, 'allow_put': False,
               'validate': {'type:regex': attr.UUID_PATTERN},
               'is_visible': True},
        'name': {'allow_post': True, 'allow_put': False,
                 'is_visible': True, 'default': ''},
        'fq_name': {'allow_post': False, 'allow_put': False,
                    'is_visible': True},
        'tenant_id': {'allow_post': True, 'allow_put': False,
                      'required_by_policy': True,
                      'is_visible': True},
        'entries': {'allow_post': True, 'allow_put': True,
                    'is_visible': True, 'default': ''},
        'nets_using': {'allow_post': False, 'allow_put': False,
                       'is_visible': True, 'default': ''},
    },
}


class Policy(object):

    @classmethod
    def get_name(cls):
        return "Network Policy"

    @classmethod
    def get_alias(cls):
        return "policy"

    @classmethod
    def get_description(cls):
        return ("Configuration object for Network Policies")

    @classmethod
    def get_namespace(cls):
        return "http://docs.openstack.org/TODO"

    @classmethod
    def get_updated(cls):
        return "2012-07-20T10:00:00-00:00"

    @classmethod
    def get_resources(cls):
        """ Returns Ext Resources """
        exts = []
        plugin = manager.NeutronManager.get_plugin()
        for resource_name in ['policy']:
            collection_name = resource_name + "s"
            params = RESOURCE_ATTRIBUTE_MAP.get(collection_name, dict())

            member_actions = {}

            controller = base.create_resource(collection_name,
                                              resource_name,
                                              plugin, params,
                                              member_actions=member_actions)

            ex = extensions.ResourceExtension(collection_name,
                                              controller,
                                              member_actions=member_actions)
            exts.append(ex)

        return exts

#end class Policy


class PolicyPluginBase(object):

    @abstractmethod
    def create_policy(self, context, policy):
        pass

    @abstractmethod
    def update_policy(self, context, id, policy):
        pass

    @abstractmethod
    def get_policy(self, context, id, fields=None):
        pass

    @abstractmethod
    def delete_policy(self, context, id):
        pass

    @abstractmethod
    def get_policys(self, context, filters=None, fields=None):
        pass
#end class PolicyPluginBase



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2014 Juniper Networks.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Hampapur Ajay, Praneet Bachheti

import copy
import logging
from pprint import pformat
import sys

import cgitb

LOG = logging.getLogger(__name__)


class NeutronPluginContrailIpam(object):
    def set_core(self, core_instance):
        self._core = core_instance

    def _make_ipam_dict(self, entry, status_code=None, fields=None):
        return entry

    def create_ipam(self, context, ipam):
        """
        Creates a new ipam, and assigns it a symbolic name.
        """
        plugin_ipam = copy.deepcopy(ipam)

        ipam_dicts = self._core._create_resource('ipam', context, plugin_ipam)
        LOG.debug("create_ipam(): " + pformat(ipam_dicts) + "\n")

        return ipam_dicts

    def get_ipam(self, context, ipam_id, fields=None):
        """
        Get the attributes of a ipam.
        """
        ipam_dicts = self._core._get_resource('ipam', context, ipam_id, fields)

        LOG.debug("get_ipam(): " + pformat(ipam_dicts))
        return ipam_dicts

    def update_ipam(self, context, ipam_id, ipam):
        """
        Updates the attributes of a particular ipam.
        """
        plugin_ipam = copy.deepcopy(ipam)
        ipam_dicts = self._core._update_resource('ipam', context, ipam_id,
                                           plugin_ipam)

        LOG.debug("update_ipam(): " + pformat(ipam_dicts))
        return ipam_dicts

    def delete_ipam(self, context, ipam_id):
        """
        Deletes the ipam with the specified identifier.
        """
        self._core._delete_resource('ipam', context, ipam_id)

        LOG.debug("delete_ipam(): %s" % (ipam_id))

    def get_ipams(self, context, filters=None, fields=None):
        """
        Retrieves all ipams identifiers.
        """
        ipam_dicts = self._core._list_resource('ipam', context, filters,
                                               fields)

        LOG.debug(
            "get_ipams(): filters: " + pformat(filters) + " data: "
            + pformat(ipam_dicts))
        return ipam_dicts

    def get_ipams_count(self, context, filters=None):
        """
        Get the count of ipams.
        """
        ipams_count = self._core._count_resource('ipam', context, filters)

        LOG.debug("get_ipams_count(): filters: " + pformat(filters) +
                  " data: " + str(ipams_count['count']))
        return ipams_count['count']



# vim: tabstop=4 shiftwidth=4 softtabstop=4

import ConfigParser
from pprint import pformat

from neutron.openstack.common import log as logging

from oslo.config import cfg
from httplib2 import Http
import re
import string
import sys
import cgitb
import uuid
import requests

from cfgm_common import exceptions as vnc_exc
from vnc_api import vnc_api

LOG = logging.getLogger(__name__)

vnc_conn = None

class QuotaDriver(object):
    """Configuration driver.

    Driver to perform necessary checks to enforce quotas and obtain
    quota information. The default driver utilizes the default values
    in neutron.conf.
    """
    quota_neutron_to_contrail_type = {
            'subnet': 'subnet',
            'network': 'virtual_network',
            'floatingip': 'floating_ip',
            'route_table': 'logical_router',
            'security_group': 'security_group',
            'security_group_rule': 'security_group_rule',
            'router': 'logical_router',
            'port': 'virtual_machine_interface',
            'pool': 'loadbalancer_pool',
            'vip': 'virtual_ip',
            'member': 'loadbalancer_member',
            'health_monitor': 'loadbalancer_healthmonitor'
            };

    @classmethod
    def _get_vnc_conn(cls):
        global vnc_conn
        if vnc_conn:
            return vnc_conn
        # Retry till a api-server is up
        while True:
            try:
                vnc_conn = vnc_api.VncApi(
                    cfg.CONF.keystone_authtoken.admin_user,
                    cfg.CONF.keystone_authtoken.admin_password,
                    cfg.CONF.keystone_authtoken.admin_tenant_name,
                    cfg.CONF.APISERVER.api_server_ip,
                    cfg.CONF.APISERVER.api_server_port,
                    auth_host=cfg.CONF.keystone_authtoken.auth_host,
                    auth_port=cfg.CONF.keystone_authtoken.auth_port,
                    auth_protocol=cfg.CONF.keystone_authtoken.auth_protocol)
                return vnc_conn
            except requests.exceptions.RequestException as e:
                time.sleep(3)
    # end _get_vnc_conn

    def limit_check(self, context, tenant_id,
                    resources, values):
        """Check simple quota limits.

        For limits--those quotas for which there is no usage
        synchronization function--this method checks that a set of
        proposed values are permitted by the limit restriction.

        This method will raise a QuotaResourceUnknown exception if a
        given resource is unknown or if it is not a simple limit
        resource.

        If any of the proposed values is over the defined quota, an
        OverQuota exception will be raised with the sorted list of the
        resources which are too high.  Otherwise, the method returns
        nothing.

        :param context: The request context, for access checks.
        :param tennant_id: The tenant_id to check quota.
        :param resources: A dictionary of the registered resources.
        :param values: A dictionary of the values to check against the
                       quota.
        """

    @classmethod
    def get_tenant_quotas(cls, context, resources, tenant_id):
        try:
            default_project = cls._get_vnc_conn().project_read(
                fq_name=['default-domain', 'default-project'])
            default_quota = default_project.get_quota()
        except vnc_exc.NoIdError:
            default_quota = None
        return cls._get_tenant_quotas(context, resources, tenant_id,
                                      default_quota)

    @classmethod
    def _get_tenant_quotas(cls, context, resources, tenant_id, default_quota):
        try:
            proj_id = str(uuid.UUID(tenant_id))
            proj_obj = cls._get_vnc_conn().project_read(id=proj_id)
            quota = proj_obj.get_quota()
        except vnc_exc.NoIdError:
            return {}
        except Exception as e:
            cgitb.Hook(format="text").handle(sys.exc_info())
            raise e

        qn2c = cls.quota_neutron_to_contrail_type
        quotas = {}
        for resource in resources:
            quota_res = None
            if quota and resource in qn2c:
                quota_res = getattr(quota, qn2c[resource], None)
            if quota_res is None and default_quota and resource in qn2c:
                quota_res = getattr(default_quota, qn2c[resource], None)
                if quota_res is None:
                    quota_res = default_quota.get_defaults()
            if quota_res is None:
                quota_res = resources[resource].default
            quotas[resource] = quota_res
        return quotas

    @classmethod
    def get_all_quotas(cls, context, resources):
        try:
            default_project = cls._get_vnc_conn().project_read(
                fq_name=['default-domain', 'default-project'])
            default_quota = default_project.get_quota()
        except vnc_exc.NoIdError:
            default_quota = None

        project_list = cls._get_vnc_conn().projects_list()['projects']
        ret_list = []
        for project in project_list:
            if default_quota and (project['uuid'] == default_project.uuid):
                continue
            quotas = cls._get_tenant_quotas(context, resources, project['uuid'],
                                            default_quota)
            quotas['tenant_id'] = project['uuid']
            ret_list.append(quotas)
        return ret_list

    @classmethod
    def delete_tenant_quota(cls, context, tenant_id):
        try:
            proj_id = str(uuid.UUID(tenant_id))
            proj_obj = cls._get_vnc_conn().project_read(id=proj_id)
            quota = proj_obj.get_quota()
        except vnc_exc.NoIdError:
            return
        except Exception as e:
            cgitb.Hook(format="text").handle(sys.exc_info())
            raise e

        for k,v in quota.__dict__.items():
            if k != 'defaults':
                quota.__dict__[k] = None
        proj_obj.set_quota(quota)
        cls._get_vnc_conn().project_update(proj_obj)

    @classmethod
    def update_quota_limit(cls, context, tenant_id, resource, limit):
        try:
            proj_id = str(uuid.UUID(tenant_id))
            proj_obj = cls._get_vnc_conn().project_read(id=proj_id)
            quota = proj_obj.get_quota() or vnc_api.QuotaType()
        except vnc_exc.NoIdError:
            return
        except Exception as e:
            cgitb.Hook(format="text").handle(sys.exc_info())
            raise e

        qn2c = cls.quota_neutron_to_contrail_type
        if resource in qn2c:
            quota_method = 'set_' + qn2c[resource]
            set_quota = getattr(quota, quota_method)
            set_quota(limit)
            proj_obj.set_quota(quota)
            cls._get_vnc_conn().project_update(proj_obj)
